"use strict";
var fs = require("fs");
var unzip = require("unzip");
var Future = require("fibers/future");
var path = require("path");

var rimraf = require("rimraf");
var hostInfo = require("./../common/host-info");

var FileSystem = (function () {
    function FileSystem() {
        this._stat = Future.wrap(fs.stat);
        this._readFile = Future.wrap(fs.readFile);
        this._writeFile = Future.wrap(fs.writeFile);
        this._readdir = Future.wrap(fs.readdir);
        this._chmod = Future.wrap(fs.chmod);
    }
    FileSystem.prototype.zipFiles = function (zipFile, files, zipPathCallback) {
        var $logger = $injector.resolve("logger");
        var zipstream = require("zipstream");
        var zip = zipstream.createZip({ level: 9 });
        var outFile = fs.createWriteStream(zipFile);
        zip.pipe(outFile);

        var result = new Future();
        outFile.on("error", function (err) {
            return result.throw(err);
        });

        var fileIdx = -1;
        var zipCallback = function () {
            fileIdx++;
            if (fileIdx < files.length) {
                var file = files[fileIdx];

                var relativePath = zipPathCallback(file);
                relativePath = relativePath.replace(/\\/g, "/");
                $logger.trace("zipping as '%s' file '%s'", relativePath, file);

                zip.addFile(fs.createReadStream(file), { name: relativePath }, zipCallback);
            } else {
                outFile.on("finish", function () {
                    return result.return();
                });

                zip.finalize(function (bytesWritten) {
                    $logger.debug("zipstream: %d bytes written", bytesWritten);
                    outFile.end();
                });
            }
        };
        zipCallback();

        return result;
    };

    FileSystem.prototype.unzip = function (zipFile, destinationDir) {
        var _this = this;
        return (function () {
            if (hostInfo.isDarwin()) {
                var $childProcess = $injector.resolve("$childProcess");

                _this.createDirectory(destinationDir).wait();
                var unzipProc = $childProcess.spawn('unzip', ['-u', '-o', zipFile, '-d', destinationDir], { stdio: "ignore", detached: true });
                _this.futureFromEvent(unzipProc, "close").wait();
            } else {
                _this.futureFromEvent(_this.createReadStream(zipFile).pipe(unzip.Extract({ path: destinationDir })), "close").wait();
            }
        }).future()();
    };

    FileSystem.prototype.exists = function (path) {
        var future = new Future();
        fs.exists(path, function (exists) {
            return future.return(exists);
        });
        return future;
    };

    FileSystem.prototype.deleteFile = function (path) {
        var future = new Future();
        fs.unlink(path, function (err) {
            if (err && err.code !== "ENOENT") {
                future.throw(err);
            } else {
                future.return();
            }
        });
        return future;
    };

    FileSystem.prototype.deleteDirectory = function (directory) {
        var future = new Future();
        rimraf(directory, function (err) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });

        return future;
    };

    FileSystem.prototype.getFileSize = function (path) {
        var _this = this;
        return (function () {
            var stat = _this.getFsStats(path).wait();
            return stat.size;
        }).future()();
    };

    FileSystem.prototype.futureFromEvent = function (eventEmitter, event) {
        var future = new Future();
        eventEmitter.once(event, function () {
            var args = _.toArray(arguments);

            if (event === "error") {
                var err = args[0];
                future.throw(err);
                return;
            }

            switch (args.length) {
                case 0:
                    future.return();
                    break;
                case 1:
                    future.return(args[0]);
                    break;
                default:
                    future.return(args);
                    break;
            }
        });
        return future;
    };

    FileSystem.prototype.createDirectory = function (path) {
        var future = new Future();
        require("mkdirp")(path, function (err) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });
        return future;
    };

    FileSystem.prototype.readDirectory = function (path) {
        return this._readdir(path);
    };

    FileSystem.prototype.readFile = function (filename) {
        return this._readFile(filename);
    };

    FileSystem.prototype.readText = function (filename, encoding) {
        return this._readFile(filename, { encoding: encoding || "utf8" });
    };

    FileSystem.prototype.readJson = function (filename, encoding) {
        var _this = this;
        return (function () {
            var data = _this.readText(filename, encoding).wait();
            if (data) {
                return JSON.parse(data);
            }
            return null;
        }).future()();
    };

    FileSystem.prototype.writeFile = function (filename, data, encoding) {
        var _this = this;
        return (function () {
            _this.createDirectory(path.dirname(filename)).wait();
            _this._writeFile(filename, data, { encoding: encoding }).wait();
        }).future()();
    };

    FileSystem.prototype.writeJson = function (filename, data, space, encoding) {
        return this.writeFile(filename, JSON.stringify(data, null, space), encoding);
    };

    FileSystem.prototype.copyFile = function (sourceFileName, destinationFileName) {
        var _this = this;
        return (function () {
            _this.createDirectory(path.dirname(destinationFileName)).wait();
            var source = _this.createReadStream(sourceFileName);
            var target = _this.createWriteStream(destinationFileName);
            source.pipe(target);
            _this.futureFromEvent(target, "finish").wait();
        }).future()();
    };

    FileSystem.prototype.createReadStream = function (path, options) {
        return fs.createReadStream(path, options);
    };

    FileSystem.prototype.createWriteStream = function (path, options) {
        return fs.createWriteStream(path, options);
    };

    FileSystem.prototype.chmod = function (path, mode) {
        return this._chmod(path, mode);
    };

    FileSystem.prototype.getFsStats = function (path) {
        return this._stat(path);
    };

    FileSystem.prototype.getUniqueFileName = function (baseName) {
        var _this = this;
        return (function () {
            if (!_this.exists(baseName).wait()) {
                return baseName;
            }
            var extension = path.extname(baseName);
            var prefix = path.basename(baseName, extension);

            for (var i = 2; ; ++i) {
                var numberedName = prefix + i + extension;
                if (!_this.exists(numberedName).wait()) {
                    return numberedName;
                }
            }
        }).future()();
    };

    FileSystem.prototype.isEmptyDir = function (directoryPath) {
        var _this = this;
        return (function () {
            var directoryContent = _this.readDirectory(directoryPath).wait();
            return directoryContent.length === 0;
        }).future()();
    };

    FileSystem.prototype.ensureDirectoryExists = function (directoryPath) {
        var _this = this;
        return (function () {
            if (!_this.exists(directoryPath).wait()) {
                _this.createDirectory(directoryPath).wait();
            }
        }).future()();
    };

    FileSystem.prototype.rename = function (oldPath, newPath) {
        var future = new Future();
        fs.rename(oldPath, newPath, function (err) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });

        return future;
    };
    return FileSystem;
})();
exports.FileSystem = FileSystem;
$injector.register("fs", FileSystem);
