"use strict";
var path = require("path");
var util = require("util");
var _ = require("underscore");
var uuid = require("node-uuid");

function createGUID(useBraces) {
    if (typeof useBraces === "undefined") { useBraces = true; }
    var output;

    if (useBraces) {
        output = '{' + uuid.v4() + '}';
    } else {
        output = uuid.v4();
    }

    return output;
}
exports.createGUID = createGUID;

function stringReplaceAll(string, find, replace) {
    return string.split(find).join(replace);
}
exports.stringReplaceAll = stringReplaceAll;

function isRequestSuccessful(request) {
    return request.statusCode >= 200 && request.statusCode < 300;
}
exports.isRequestSuccessful = isRequestSuccessful;

function isResponseRedirect(response) {
    return _.contains([301, 302, 303, 307, 308], response.statusCode);
}
exports.isResponseRedirect = isResponseRedirect;

function enumerateFilesInDirectorySyncRecursive(foundFiles, directoryPath, filterCallback) {
    var $fs = $injector.resolve("fs");
    var contents = $fs.readDirectory(directoryPath).wait();
    for (var i = 0; i < contents.length; ++i) {
        var file = path.join(directoryPath, contents[i]);
        var stat = $fs.getFsStats(file).wait();
        if (filterCallback && !filterCallback(file, stat)) {
            continue;
        }

        if (stat.isDirectory()) {
            enumerateFilesInDirectorySyncRecursive(foundFiles, file, filterCallback);
        } else {
            foundFiles.push(file);
        }
    }
}

function enumerateFilesInDirectorySync(directoryPath, filterCallback) {
    var result = [];
    enumerateFilesInDirectorySyncRecursive(result, directoryPath, filterCallback);
    return result;
}
exports.enumerateFilesInDirectorySync = enumerateFilesInDirectorySync;

function getParsedOptions(options, shorthands) {
    var yargs = require("yargs");
    Object.keys(options).forEach(function (opt) {
        var type = options[opt];
        if (type === String) {
            yargs.string(opt);
        } else if (type === Boolean) {
            yargs.boolean(opt);
        }
    });

    Object.keys(shorthands).forEach(function (key) {
        yargs.alias(key, shorthands[key]);
    });

    var parsed = yargs.argv;
    exports.validateYargsArguments(parsed, options, shorthands);

    return parsed;
}
exports.getParsedOptions = getParsedOptions;

function validateYargsArguments(parsed, knownOpts, shorthands, isInTestMode) {
    if (path.basename(process.argv[1]) === "appbuilder.js" || isInTestMode) {
        _.each(_.keys(parsed), function (opt) {
            var option = shorthands[opt] ? shorthands[opt] : opt;

            if (option !== "_" && option !== "$0" && !knownOpts[option]) {
                exports.breakExecution(util.format("The option '%s' is not supported. To see command's options, use '$ appbuilder %s --help'. To see all commands use '$ appbuilder help'.", opt, process.argv[2]));
            } else if (knownOpts[option] !== Boolean && typeof (parsed[opt]) === 'boolean') {
                exports.breakExecution(util.format("The option '%s' requires a value.", opt));
            } else if (knownOpts[option] === String && parsed[opt].trim().length === 0) {
                exports.breakExecution(util.format("The option '%s' requires nonempty value.", opt));
            } else if (knownOpts[option] === Boolean && typeof (parsed[opt]) !== 'boolean') {
                exports.breakExecution(util.format("The option '%s' does not accept values.", opt));
            }
        });
    }
}
exports.validateYargsArguments = validateYargsArguments;

function breakExecution(message) {
    console.log("\x1B[31;1m" + message + "\x1B[0m");
    process.exit(128 /* INVALID_ARGUMENT */);
}
exports.breakExecution = breakExecution;

function formatListOfNames(names, conjunction) {
    if (typeof conjunction === "undefined") { conjunction = "or"; }
    if (names.length <= 1) {
        return names[0];
    } else {
        return _.initial(names).join(", ") + " " + conjunction + " " + names[names.length - 1];
    }
}
exports.formatListOfNames = formatListOfNames;

function getRelativeToRootPath(rootPath, filePath) {
    var relativeToRootPath = filePath.substr(rootPath.length);
    return relativeToRootPath;
}
exports.getRelativeToRootPath = getRelativeToRootPath;

function versionCompare(version1, version2) {
    version1 = version1.split("-")[0];
    version2 = version2.split("-")[0];
    var v1array = _.map(version1.split("."), function (x) {
        return parseInt(x, 10);
    }), v2array = _.map(version2.split("."), function (x) {
        return parseInt(x, 10);
    });

    if (v1array.length !== v2array.length) {
        throw new Error("Version strings are not in the same format");
    }

    for (var i = 0; i < v1array.length; ++i) {
        if (v1array[i] !== v2array[i]) {
            return v1array[i] > v2array[i] ? 1 : -1;
        }
    }

    return 0;
}
exports.versionCompare = versionCompare;

function isInteractive() {
    return process.stdout.isTTY && process.stdin.isTTY;
}
exports.isInteractive = isInteractive;

function toBoolean(str) {
    return str === "true";
}
exports.toBoolean = toBoolean;

function registerCommand(module, commandName, executor, opts) {
    var factory = function () {
        return {
            execute: function (args) {
                var mod = $injector.resolve(module);
                return executor(mod, args);
            },
            disableAnalytics: opts && opts.disableAnalytics
        };
    };

    $injector.registerCommand(commandName, factory);
}
exports.registerCommand = registerCommand;

function block(operation) {
    if (exports.isInteractive()) {
        process.stdin.setRawMode(false);
    }
    operation();
    if (exports.isInteractive()) {
        process.stdin.setRawMode(true);
    }
}
exports.block = block;

function isNumber(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
}
exports.isNumber = isNumber;

function fromWindowsRelativePathToUnix(windowsRelativePath) {
    return windowsRelativePath.replace(/\\/g, "/");
}
exports.fromWindowsRelativePathToUnix = fromWindowsRelativePathToUnix;

function isNullOrWhitespace(input) {
    if (!input) {
        return true;
    }

    return input.replace(/\s/gi, '').length < 1;
}
exports.isNullOrWhitespace = isNullOrWhitespace;
