"use strict";
var path = require("path");
var ref = require("ref");
var ffi = require("ffi");
var struct = require("ref-struct");
var bufferpack = require("bufferpack");
var plistlib = require("plistlib");
var helpers = require("../../helpers");
var hostInfo = require("../../host-info");
var net = require("net");

var Future = require("fibers/future");

var CoreTypes = (function () {
    function CoreTypes() {
    }
    CoreTypes.pointerSize = ref.types.size_t.size;
    CoreTypes.voidPtr = ref.refType(ref.types.void);
    CoreTypes.intPtr = ref.refType(ref.types.int);
    CoreTypes.uintPtr = ref.refType(ref.types.uint);
    CoreTypes.charPtr = ref.refType(ref.types.char);
    CoreTypes.ptrToVoidPtr = ref.refType(ref.refType(ref.types.void));
    CoreTypes.uintType = ref.types.uint;
    CoreTypes.uint32Type = ref.types.uint32;
    CoreTypes.intType = ref.types.int;
    CoreTypes.boolType = ref.types.bool;
    CoreTypes.doubleType = ref.types.double;

    CoreTypes.am_device_p = CoreTypes.voidPtr;
    CoreTypes.cfDictionaryRef = CoreTypes.voidPtr;
    CoreTypes.cfDataRef = CoreTypes.voidPtr;
    CoreTypes.cfStringRef = CoreTypes.voidPtr;
    CoreTypes.afcConnectionRef = CoreTypes.voidPtr;
    CoreTypes.afcFileRef = ref.types.uint64;
    CoreTypes.afcDirectoryRef = CoreTypes.voidPtr;
    CoreTypes.afcError = ref.types.uint32;
    CoreTypes.amDeviceRef = CoreTypes.voidPtr;
    CoreTypes.amDeviceNotificationRef = CoreTypes.voidPtr;
    CoreTypes.cfTimeInterval = ref.types.double;
    CoreTypes.kCFPropertyListXMLFormat_v1_0 = 100;
    CoreTypes.kCFPropertyListBinaryFormat_v1_0 = 200;
    CoreTypes.kCFPropertyListImmutable = 0;

    CoreTypes.am_device_notification = struct({
        unknown0: ref.types.uint32,
        unknown1: ref.types.uint32,
        unknown2: ref.types.uint32,
        callback: CoreTypes.voidPtr,
        cookie: ref.types.uint32
    });

    CoreTypes.am_device_notification_callback_info = struct({
        dev: CoreTypes.am_device_p,
        msg: ref.types.uint,
        subscription: ref.refType(CoreTypes.am_device_notification)
    });

    CoreTypes.am_device_notification_callback = ffi.Function("void", [ref.refType(CoreTypes.am_device_notification_callback_info), CoreTypes.voidPtr]);
    CoreTypes.am_device_install_application_callback = ffi.Function("void", [CoreTypes.cfDictionaryRef, CoreTypes.voidPtr]);
    CoreTypes.cf_run_loop_timer_callback = ffi.Function("void", [CoreTypes.voidPtr, CoreTypes.voidPtr]);
    return CoreTypes;
})();
exports.CoreTypes = CoreTypes;

var IOSCore = (function () {
    function IOSCore($logger, $fs, $errors) {
        this.$logger = $logger;
        this.$fs = $fs;
        this.$errors = $errors;
        this.cfDictionaryKeyCallBacks = struct({
            version: CoreTypes.uintType,
            retain: CoreTypes.voidPtr,
            release: CoreTypes.voidPtr,
            copyDescription: CoreTypes.voidPtr,
            equal: CoreTypes.voidPtr,
            hash: CoreTypes.voidPtr
        });
        this.cfDictionaryValueCallBacks = struct({
            version: CoreTypes.uintType,
            retain: CoreTypes.voidPtr,
            release: CoreTypes.voidPtr,
            copyDescription: CoreTypes.voidPtr,
            equal: CoreTypes.voidPtr
        });
    }
    Object.defineProperty(IOSCore.prototype, "CoreFoundationDir", {
        get: function () {
            if (hostInfo.isWindows()) {
                return path.join(this.CommonProgramFilesPath, "Apple", "Apple Application Support");
            } else if (hostInfo.isDarwin()) {
                return "/System/Library/Frameworks/CoreFoundation.framework/CoreFoundation";
            }

            return null;
        },
        enumerable: true,
        configurable: true
    });

    Object.defineProperty(IOSCore.prototype, "MobileDeviceDir", {
        get: function () {
            if (hostInfo.isWindows()) {
                return path.join(this.CommonProgramFilesPath, "Apple", "Mobile Device Support");
            } else if (hostInfo.isDarwin()) {
                return "/System/Library/PrivateFrameworks/MobileDevice.framework/MobileDevice";
            }

            return null;
        },
        enumerable: true,
        configurable: true
    });

    Object.defineProperty(IOSCore.prototype, "CommonProgramFilesPath", {
        get: function () {
            return hostInfo.isWindows64() ? process.env["CommonProgramFiles(x86)"] : process.env.CommonProgramFiles;
        },
        enumerable: true,
        configurable: true
    });

    IOSCore.prototype.is32BitProcess = function () {
        return ref.types.size_t.size === 4;
    };

    IOSCore.prototype.getForeignPointer = function (lib, name, type) {
        var pointer = lib.get(name);
        pointer.type = ref.refType(type);
        return pointer;
    };

    IOSCore.prototype.getCoreFoundationLibrary = function () {
        if (hostInfo.isWindows()) {
            process.env.PATH = this.CoreFoundationDir + ";" + process.env.PATH;
            process.env.PATH += ";" + this.MobileDeviceDir;
        }

        var coreFoundationDll = hostInfo.isWindows() ? path.join(this.CoreFoundationDir, "CoreFoundation.dll") : this.CoreFoundationDir;
        var lib = ffi.DynamicLibrary(coreFoundationDll);

        return {
            "CFRunLoopRun": ffi.ForeignFunction(lib.get("CFRunLoopRun"), "void", []),
            "CFRunLoopStop": ffi.ForeignFunction(lib.get("CFRunLoopStop"), "void", [CoreTypes.voidPtr]),
            "CFRunLoopGetCurrent": ffi.ForeignFunction(lib.get("CFRunLoopGetCurrent"), CoreTypes.voidPtr, []),
            "CFStringCreateWithCString": ffi.ForeignFunction(lib.get("CFStringCreateWithCString"), CoreTypes.cfStringRef, [CoreTypes.voidPtr, "string", "uint"]),
            "CFDictionaryGetValue": ffi.ForeignFunction(lib.get("CFDictionaryGetValue"), CoreTypes.voidPtr, [CoreTypes.cfDictionaryRef, CoreTypes.cfStringRef]),
            "CFNumberGetValue": ffi.ForeignFunction(lib.get("CFNumberGetValue"), CoreTypes.boolType, [CoreTypes.voidPtr, "uint", CoreTypes.voidPtr]),
            "CFStringGetCStringPtr": ffi.ForeignFunction(lib.get("CFStringGetCStringPtr"), CoreTypes.charPtr, [CoreTypes.cfStringRef, "uint"]),
            "CFStringGetCString": ffi.ForeignFunction(lib.get("CFStringGetCString"), CoreTypes.boolType, [CoreTypes.cfStringRef, CoreTypes.charPtr, "uint", "uint"]),
            "CFStringGetLength": ffi.ForeignFunction(lib.get("CFStringGetLength"), "ulong", [CoreTypes.cfStringRef]),
            "CFDictionaryGetCount": ffi.ForeignFunction(lib.get("CFDictionaryGetCount"), "int", [CoreTypes.cfDictionaryRef]),
            "CFDictionaryGetKeysAndValues": ffi.ForeignFunction(lib.get("CFDictionaryGetKeysAndValues"), "void", [CoreTypes.cfDictionaryRef, CoreTypes.ptrToVoidPtr, CoreTypes.ptrToVoidPtr]),
            "CFDictionaryCreate": ffi.ForeignFunction(lib.get("CFDictionaryCreate"), CoreTypes.cfDictionaryRef, [CoreTypes.voidPtr, CoreTypes.ptrToVoidPtr, CoreTypes.ptrToVoidPtr, "int", ref.refType(this.cfDictionaryKeyCallBacks), ref.refType(this.cfDictionaryValueCallBacks)]),
            "kCFTypeDictionaryKeyCallBacks": lib.get("kCFTypeDictionaryKeyCallBacks"),
            "kCFTypeDictionaryValueCallBacks": lib.get("kCFTypeDictionaryValueCallBacks"),
            "CFRunLoopRunInMode": ffi.ForeignFunction(lib.get("CFRunLoopRunInMode"), CoreTypes.intType, [CoreTypes.cfStringRef, CoreTypes.cfTimeInterval, CoreTypes.boolType]),
            "kCFRunLoopDefaultMode": this.getForeignPointer(lib, "kCFRunLoopDefaultMode", ref.types.void),
            "kCFRunLoopCommonModes": this.getForeignPointer(lib, "kCFRunLoopCommonModes", ref.types.void),
            "CFRunLoopTimerCreate": ffi.ForeignFunction(lib.get("CFRunLoopTimerCreate"), CoreTypes.voidPtr, [CoreTypes.voidPtr, CoreTypes.doubleType, CoreTypes.doubleType, CoreTypes.uintType, CoreTypes.uintType, CoreTypes.cf_run_loop_timer_callback, CoreTypes.voidPtr]),
            "CFRunLoopAddTimer": ffi.ForeignFunction(lib.get("CFRunLoopAddTimer"), "void", [CoreTypes.voidPtr, CoreTypes.voidPtr, CoreTypes.cfStringRef]),
            "CFRunLoopRemoveTimer": ffi.ForeignFunction(lib.get("CFRunLoopRemoveTimer"), "void", [CoreTypes.voidPtr, CoreTypes.voidPtr, CoreTypes.cfStringRef]),
            "CFAbsoluteTimeGetCurrent": ffi.ForeignFunction(lib.get("CFAbsoluteTimeGetCurrent"), CoreTypes.doubleType, []),
            "CFPropertyListCreateData": ffi.ForeignFunction(lib.get("CFPropertyListCreateData"), CoreTypes.voidPtr, [CoreTypes.voidPtr, CoreTypes.voidPtr, ref.types.long, ref.types.ulong, CoreTypes.voidPtr]),
            "CFPropertyListCreateWithData": ffi.ForeignFunction(lib.get("CFPropertyListCreateWithData"), CoreTypes.voidPtr, [CoreTypes.voidPtr, CoreTypes.voidPtr, ref.types.ulong, ref.refType(ref.types.long), CoreTypes.voidPtr]),
            "CFGetTypeID": ffi.ForeignFunction(lib.get("CFGetTypeID"), ref.types.long, [CoreTypes.voidPtr]),
            "CFStringGetTypeID": ffi.ForeignFunction(lib.get("CFStringGetTypeID"), ref.types.long, []),
            "CFDictionaryGetTypeID": ffi.ForeignFunction(lib.get("CFDictionaryGetTypeID"), ref.types.long, []),
            "CFDataGetTypeID": ffi.ForeignFunction(lib.get("CFDataGetTypeID"), ref.types.long, []),
            "CFNumberGetTypeID": ffi.ForeignFunction(lib.get("CFNumberGetTypeID"), ref.types.long, []),
            "CFBooleanGetTypeID": ffi.ForeignFunction(lib.get("CFBooleanGetTypeID"), ref.types.long, []),
            "CFArrayGetTypeID": ffi.ForeignFunction(lib.get("CFArrayGetTypeID"), ref.types.long, []),
            "CFDateGetTypeID": ffi.ForeignFunction(lib.get("CFDateGetTypeID"), ref.types.long, []),
            "CFSetGetTypeID": ffi.ForeignFunction(lib.get("CFSetGetTypeID"), ref.types.long, []),
            "CFDataGetBytePtr": ffi.ForeignFunction(lib.get("CFDataGetBytePtr"), ref.refType(ref.types.uint8), [CoreTypes.voidPtr]),
            "CFDataGetLength": ffi.ForeignFunction(lib.get("CFDataGetLength"), ref.types.long, [CoreTypes.voidPtr]),
            "CFDataCreate": ffi.ForeignFunction(lib.get("CFDataCreate"), CoreTypes.voidPtr, [CoreTypes.voidPtr, CoreTypes.voidPtr, ref.types.long])
        };
    };

    IOSCore.prototype.getMobileDeviceLibrary = function () {
        var mobileDeviceDll = hostInfo.isWindows() ? path.join(this.MobileDeviceDir, "MobileDevice.dll") : this.MobileDeviceDir;
        var lib = ffi.DynamicLibrary(mobileDeviceDll);

        return {
            "AMDeviceNotificationSubscribe": ffi.ForeignFunction(lib.get("AMDeviceNotificationSubscribe"), "uint", [CoreTypes.am_device_notification_callback, "uint", "uint", "uint", CoreTypes.ptrToVoidPtr]),
            "AMDeviceConnect": ffi.ForeignFunction(lib.get("AMDeviceConnect"), "uint", [CoreTypes.am_device_p]),
            "AMDeviceIsPaired": ffi.ForeignFunction(lib.get("AMDeviceIsPaired"), "uint", [CoreTypes.am_device_p]),
            "AMDevicePair": ffi.ForeignFunction(lib.get("AMDevicePair"), "uint", [CoreTypes.am_device_p]),
            "AMDeviceValidatePairing": ffi.ForeignFunction(lib.get("AMDeviceValidatePairing"), "uint", [CoreTypes.am_device_p]),
            "AMDeviceStartSession": ffi.ForeignFunction(lib.get("AMDeviceStartSession"), "uint", [CoreTypes.am_device_p]),
            "AMDeviceStopSession": ffi.ForeignFunction(lib.get("AMDeviceStopSession"), "uint", [CoreTypes.am_device_p]),
            "AMDeviceDisconnect": ffi.ForeignFunction(lib.get("AMDeviceDisconnect"), "uint", [CoreTypes.am_device_p]),
            "AMDeviceStartService": ffi.ForeignFunction(lib.get("AMDeviceStartService"), "uint", [CoreTypes.am_device_p, CoreTypes.cfStringRef, CoreTypes.intPtr, CoreTypes.voidPtr]),
            "AMDeviceTransferApplication": ffi.ForeignFunction(lib.get("AMDeviceTransferApplication"), "uint", ["int", CoreTypes.cfStringRef, CoreTypes.cfDictionaryRef, CoreTypes.am_device_install_application_callback, CoreTypes.voidPtr]),
            "AMDeviceInstallApplication": ffi.ForeignFunction(lib.get("AMDeviceInstallApplication"), "uint", ["int", CoreTypes.cfStringRef, CoreTypes.cfDictionaryRef, CoreTypes.am_device_install_application_callback, CoreTypes.voidPtr]),
            "AMDeviceLookupApplications": ffi.ForeignFunction(lib.get("AMDeviceLookupApplications"), "uint", [CoreTypes.am_device_p, "uint", ref.refType(CoreTypes.cfDictionaryRef)]),
            "AMDeviceUninstallApplication": ffi.ForeignFunction(lib.get("AMDeviceUninstallApplication"), "uint", ["int", CoreTypes.cfStringRef, CoreTypes.cfDictionaryRef, CoreTypes.am_device_install_application_callback, CoreTypes.voidPtr]),
            "AFCConnectionOpen": ffi.ForeignFunction(lib.get("AFCConnectionOpen"), "uint", ["int", "uint", ref.refType(CoreTypes.afcConnectionRef)]),
            "AFCConnectionClose": ffi.ForeignFunction(lib.get("AFCConnectionClose"), "uint", [CoreTypes.afcConnectionRef]),
            "AFCDirectoryCreate": ffi.ForeignFunction(lib.get("AFCDirectoryCreate"), "uint", [CoreTypes.afcConnectionRef, "string"]),
            "AFCFileRefOpen": hostInfo.isDarwin() ? ffi.ForeignFunction(lib.get("AFCFileRefOpen"), "uint", [CoreTypes.afcConnectionRef, "string", "uint", ref.refType(CoreTypes.afcFileRef)]) : ffi.ForeignFunction(lib.get("AFCFileRefOpen"), "uint", [CoreTypes.afcConnectionRef, "string", "uint", "uint", ref.refType(CoreTypes.afcFileRef)]),
            "AFCFileRefClose": ffi.ForeignFunction(lib.get("AFCFileRefClose"), "uint", [CoreTypes.afcConnectionRef, CoreTypes.afcFileRef]),
            "AFCFileRefWrite": ffi.ForeignFunction(lib.get("AFCFileRefWrite"), "uint", [CoreTypes.afcConnectionRef, CoreTypes.afcFileRef, CoreTypes.voidPtr, "uint"]),
            "AFCFileRefRead": ffi.ForeignFunction(lib.get("AFCFileRefRead"), "uint", [CoreTypes.afcConnectionRef, CoreTypes.afcFileRef, CoreTypes.voidPtr, CoreTypes.uintPtr]),
            "AFCRemovePath": ffi.ForeignFunction(lib.get("AFCRemovePath"), "uint", [CoreTypes.afcConnectionRef, "string"]),
            "AFCDirectoryOpen": ffi.ForeignFunction(lib.get("AFCDirectoryOpen"), CoreTypes.afcError, [CoreTypes.afcConnectionRef, "string", ref.refType(CoreTypes.afcDirectoryRef)]),
            "AFCDirectoryRead": ffi.ForeignFunction(lib.get("AFCDirectoryRead"), CoreTypes.afcError, [CoreTypes.afcConnectionRef, CoreTypes.afcDirectoryRef, ref.refType(CoreTypes.charPtr)]),
            "AFCDirectoryClose": ffi.ForeignFunction(lib.get("AFCDirectoryClose"), CoreTypes.afcError, [CoreTypes.afcConnectionRef, CoreTypes.afcDirectoryRef]),
            "AMDeviceCopyDeviceIdentifier": ffi.ForeignFunction(lib.get("AMDeviceCopyDeviceIdentifier"), CoreTypes.cfStringRef, [CoreTypes.am_device_p]),
            "AMDeviceCopyValue": ffi.ForeignFunction(lib.get("AMDeviceCopyValue"), CoreTypes.cfStringRef, [CoreTypes.am_device_p, CoreTypes.cfStringRef, CoreTypes.cfStringRef]),
            "AMDeviceNotificationUnsubscribe": ffi.ForeignFunction(lib.get("AMDeviceNotificationUnsubscribe"), CoreTypes.intType, [CoreTypes.amDeviceNotificationRef])
        };
    };

    IOSCore.getWinSocketLibrary = function () {
        var winSocketDll = path.join(process.env.SystemRoot, "System32", "ws2_32.dll");

        return ffi.Library(winSocketDll, {
            "closesocket": ["int", ["uint"]],
            "recv": ["int", ["uint", CoreTypes.charPtr, "int", "int"]],
            "send": ["int", ["uint", "string", "int", "int"]],
            "setsockopt": ["int", ["uint", "int", "int", CoreTypes.voidPtr, "int"]]
        });
    };
    IOSCore.kCFStringEncodingUTF8 = 0x08000100;
    return IOSCore;
})();
$injector.register("iOSCore", IOSCore);

var CoreFoundation = (function () {
    function CoreFoundation($iOSCore, $errors) {
        this.$errors = $errors;
        this.coreFoundationLibrary = $iOSCore.getCoreFoundationLibrary();
    }
    CoreFoundation.prototype.runLoopRun = function () {
        this.coreFoundationLibrary.CFRunLoopRun();
    };

    CoreFoundation.prototype.runLoopGetCurrent = function () {
        return this.coreFoundationLibrary.CFRunLoopGetCurrent();
    };

    CoreFoundation.prototype.kCFRunLoopCommonModes = function () {
        return this.coreFoundationLibrary.kCFRunLoopCommonModes.deref();
    };

    CoreFoundation.prototype.kCFRunLoopDefaultMode = function () {
        return this.coreFoundationLibrary.kCFRunLoopDefaultMode.deref();
    };

    CoreFoundation.prototype.kCFTypeDictionaryValueCallBacks = function () {
        return this.coreFoundationLibrary.kCFTypeDictionaryValueCallBacks;
    };

    CoreFoundation.prototype.kCFTypeDictionaryKeyCallBacks = function () {
        return this.coreFoundationLibrary.kCFTypeDictionaryKeyCallBacks;
    };

    CoreFoundation.prototype.runLoopTimerCreate = function (allocator, fireDate, interval, flags, order, callout, context) {
        return this.coreFoundationLibrary.CFRunLoopTimerCreate(allocator, fireDate, interval, flags, order, callout, context);
    };

    CoreFoundation.prototype.absoluteTimeGetCurrent = function () {
        return this.coreFoundationLibrary.CFAbsoluteTimeGetCurrent();
    };

    CoreFoundation.prototype.runLoopAddTimer = function (r1, timer, mode) {
        this.coreFoundationLibrary.CFRunLoopAddTimer(r1, timer, mode);
    };

    CoreFoundation.prototype.runLoopRemoveTimer = function (r1, timer, mode) {
        this.coreFoundationLibrary.CFRunLoopRemoveTimer(r1, timer, mode);
    };

    CoreFoundation.prototype.runLoopStop = function (r1) {
        this.coreFoundationLibrary.CFRunLoopStop(r1);
    };

    CoreFoundation.prototype.stringGetCStringPtr = function (theString, encoding) {
        return this.coreFoundationLibrary.CFStringGetCStringPtr(theString, encoding);
    };

    CoreFoundation.prototype.stringGetLength = function (theString) {
        return this.coreFoundationLibrary.CFStringGetLength(theString);
    };

    CoreFoundation.prototype.stringGetCString = function (theString, buffer, bufferSize, encoding) {
        return this.coreFoundationLibrary.CFStringGetCString(theString, buffer, bufferSize, encoding);
    };

    CoreFoundation.prototype.stringCreateWithCString = function (alloc, str, encoding) {
        return this.coreFoundationLibrary.CFStringCreateWithCString(alloc, str, encoding);
    };

    CoreFoundation.prototype.createCFString = function (str) {
        return this.stringCreateWithCString(null, str, IOSCore.kCFStringEncodingUTF8);
    };

    CoreFoundation.prototype.dictionaryCreate = function (allocator, keys, values, count, dictionaryKeyCallbacks, dictionaryValueCallbacks) {
        return this.coreFoundationLibrary.CFDictionaryCreate(allocator, keys, values, count, dictionaryKeyCallbacks, dictionaryValueCallbacks);
    };

    CoreFoundation.prototype.dictionaryGetValue = function (theDict, value) {
        return this.coreFoundationLibrary.CFDictionaryGetValue(theDict, value);
    };

    CoreFoundation.prototype.dictionaryGetCount = function (theDict) {
        return this.coreFoundationLibrary.CFDictionaryGetCount(theDict);
    };

    CoreFoundation.prototype.dictionaryGetKeysAndValues = function (dictionary, keys, values) {
        this.coreFoundationLibrary.CFDictionaryGetKeysAndValues(dictionary, keys, values);
    };

    CoreFoundation.prototype.dictionaryGetTypeID = function () {
        return this.coreFoundationLibrary.CFDictionaryGetTypeID();
    };

    CoreFoundation.prototype.numberGetValue = function (number, theType, valuePtr) {
        return this.coreFoundationLibrary.CFNumberGetValue(number, theType, valuePtr);
    };

    CoreFoundation.prototype.getTypeID = function (buffer) {
        return this.coreFoundationLibrary.CFGetTypeID(buffer);
    };

    CoreFoundation.prototype.propertyListCreateData = function (allocator, propertyListRef, propertyListFormat, optionFlags, error) {
        return this.coreFoundationLibrary.CFPropertyListCreateData(allocator, propertyListRef, propertyListFormat, optionFlags, error);
    };

    CoreFoundation.prototype.propertyListCreateWithData = function (allocator, propertyList, optionFlags, propertyListFormat, error) {
        return this.coreFoundationLibrary.CFPropertyListCreateWithData(allocator, propertyList, optionFlags, propertyListFormat, error);
    };

    CoreFoundation.prototype.stringGetTypeID = function () {
        return this.coreFoundationLibrary.CFStringGetTypeID();
    };

    CoreFoundation.prototype.dataGetTypeID = function () {
        return this.coreFoundationLibrary.CFDataGetTypeID();
    };

    CoreFoundation.prototype.numberGetTypeID = function () {
        return this.coreFoundationLibrary.CFNumberGetTypeID();
    };

    CoreFoundation.prototype.booleanGetTypeID = function () {
        return this.coreFoundationLibrary.CFBooleanGetTypeID();
    };

    CoreFoundation.prototype.arrayGetTypeID = function () {
        return this.coreFoundationLibrary.CFArrayGetTypeID();
    };

    CoreFoundation.prototype.dateGetTypeID = function () {
        return this.coreFoundationLibrary.CFDateGetTypeID();
    };

    CoreFoundation.prototype.setGetTypeID = function () {
        return this.coreFoundationLibrary.CFSetGetTypeID();
    };

    CoreFoundation.prototype.dataGetBytePtr = function (buffer) {
        return this.coreFoundationLibrary.CFDataGetBytePtr(buffer);
    };

    CoreFoundation.prototype.dataGetLength = function (buffer) {
        return this.coreFoundationLibrary.CFDataGetLength(buffer);
    };

    CoreFoundation.prototype.dataCreate = function (allocator, data, length) {
        return this.coreFoundationLibrary.CFDataCreate(allocator, data, length);
    };

    CoreFoundation.prototype.convertCFStringToCString = function (cfstr) {
        var result;
        if (cfstr != null) {
            var rawData = this.stringGetCStringPtr(cfstr, IOSCore.kCFStringEncodingUTF8);
            if (ref.address(rawData) === 0) {
                var cfstrLength = this.stringGetLength(cfstr);
                var length = cfstrLength + 1;
                var stringBuffer = new Buffer(length);
                var status = this.stringGetCString(cfstr, stringBuffer, length, IOSCore.kCFStringEncodingUTF8);
                if (status) {
                    result = stringBuffer.toString("utf8", 0, cfstrLength);
                } else {
                    this.$errors.fail("Unable to convert string: ", result);
                }
            } else {
                result = ref.readCString(rawData, 0);
            }
        }

        return result;
    };

    CoreFoundation.prototype.cfTypeFrom = function (value) {
        var keys = _.keys(value);
        var values = _.values(value);

        var len = keys.length;
        var keysBuffer = new Buffer(CoreTypes.pointerSize * len);
        var valuesBuffer = new Buffer(CoreTypes.pointerSize * len);

        var offset = 0;

        for (var i = 0; i < len; i++) {
            var cfKey = this.createCFString(keys[i]);
            var cfValue = this.createCFString(values[i]);

            ref.writePointer(keysBuffer, offset, cfKey);
            ref.writePointer(valuesBuffer, offset, cfValue);
            offset += CoreTypes.pointerSize;
        }

        return this.dictionaryCreate(null, keysBuffer, valuesBuffer, len, this.kCFTypeDictionaryKeyCallBacks(), this.kCFTypeDictionaryValueCallBacks());
    };

    CoreFoundation.prototype.cfTypeTo = function (value) {
        var typeId = this.getTypeID(value);
        var retval = null;

        if (typeId === this.dataGetTypeID()) {
            var len = this.dataGetLength(value);
            retval = ref.reinterpret(this.dataGetBytePtr(value), len);
        }

        return retval;
    };

    CoreFoundation.prototype.dictToPlistEncoding = function (dict, format) {
        var cfDict = this.cfTypeFrom(dict);
        var cfData = this.propertyListCreateData(null, cfDict, format, 0, null);

        return this.cfTypeTo(cfData);
    };

    CoreFoundation.prototype.dictFromPlistEncoding = function (str) {
        var retval = null;

        var cfData = this.dataCreate(null, str, str.length);
        if (cfData) {
            var cfDict = this.propertyListCreateWithData(null, cfData, CoreTypes.kCFPropertyListImmutable, null, null);
            if (cfDict) {
                retval = this.cfTypeTo(cfDict);
            }
        }

        return retval;
    };
    return CoreFoundation;
})();
exports.CoreFoundation = CoreFoundation;
$injector.register("coreFoundation", CoreFoundation);

var MobileDevice = (function () {
    function MobileDevice($iOSCore, $errors) {
        this.$errors = $errors;
        this.mobileDeviceLibrary = $iOSCore.getMobileDeviceLibrary();
    }
    MobileDevice.prototype.deviceNotificationSubscribe = function (notificationCallback, p1, p2, p3, callbackSignature) {
        return this.mobileDeviceLibrary.AMDeviceNotificationSubscribe(notificationCallback, p1, p2, p3, callbackSignature);
    };

    MobileDevice.prototype.deviceCopyDeviceIdentifier = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceCopyDeviceIdentifier(devicePointer);
    };

    MobileDevice.prototype.deviceCopyValue = function (devicePointer, domain, name) {
        return this.mobileDeviceLibrary.AMDeviceCopyValue(devicePointer, domain, name);
    };

    MobileDevice.prototype.deviceConnect = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceConnect(devicePointer);
    };

    MobileDevice.prototype.deviceIsPaired = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceIsPaired(devicePointer);
    };

    MobileDevice.prototype.devicePair = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDevicePair(devicePointer);
    };

    MobileDevice.prototype.deviceValidatePairing = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceValidatePairing(devicePointer);
    };

    MobileDevice.prototype.deviceStartSession = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceStartSession(devicePointer);
    };

    MobileDevice.prototype.deviceStopSession = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceStopSession(devicePointer);
    };

    MobileDevice.prototype.deviceDisconnect = function (devicePointer) {
        return this.mobileDeviceLibrary.AMDeviceDisconnect(devicePointer);
    };

    MobileDevice.prototype.deviceStartService = function (devicePointer, serviceName, socketNumber) {
        return this.mobileDeviceLibrary.AMDeviceStartService(devicePointer, serviceName, socketNumber, null);
    };

    MobileDevice.prototype.deviceTransferApplication = function (service, packageFile, options, installationCallback) {
        return this.mobileDeviceLibrary.AMDeviceTransferApplication(service, packageFile, options, installationCallback, null);
    };

    MobileDevice.prototype.deviceInstallApplication = function (service, packageFile, options, installationCallback) {
        return this.mobileDeviceLibrary.AMDeviceInstallApplication(service, packageFile, options, installationCallback, null);
    };

    MobileDevice.prototype.afcConnectionOpen = function (service, timeout, afcConnection) {
        return this.mobileDeviceLibrary.AFCConnectionOpen(service, timeout, afcConnection);
    };

    MobileDevice.prototype.afcConnectionClose = function (afcConnection) {
        return this.mobileDeviceLibrary.AFCConnectionClose(afcConnection);
    };

    MobileDevice.prototype.afcDirectoryCreate = function (afcConnection, path) {
        return this.mobileDeviceLibrary.AFCDirectoryCreate(afcConnection, path);
    };

    MobileDevice.prototype.afcFileRefOpen = function (afcConnection, path, mode, afcFileRef) {
        if (hostInfo.isWindows()) {
            return this.mobileDeviceLibrary.AFCFileRefOpen(afcConnection, path, mode, 0, afcFileRef);
        } else if (hostInfo.isDarwin()) {
            return this.mobileDeviceLibrary.AFCFileRefOpen(afcConnection, path, mode, afcFileRef);
        }
    };

    MobileDevice.prototype.afcFileRefClose = function (afcConnection, afcFileRef) {
        return this.mobileDeviceLibrary.AFCFileRefClose(afcConnection, afcFileRef);
    };

    MobileDevice.prototype.afcFileRefWrite = function (afcConnection, afcFileRef, buffer, byteLength) {
        return this.mobileDeviceLibrary.AFCFileRefWrite(afcConnection, afcFileRef, buffer, byteLength);
    };

    MobileDevice.prototype.afcFileRefRead = function (afcConnection, afcFileRef, buffer, byteLength) {
        return this.mobileDeviceLibrary.AFCFileRefRead(afcConnection, afcFileRef, buffer, byteLength);
    };

    MobileDevice.prototype.afcRemovePath = function (afcConnection, path) {
        return this.mobileDeviceLibrary.AFCRemovePath(afcConnection, path);
    };

    MobileDevice.prototype.afcDirectoryOpen = function (afcConnection, path, afcDirectory) {
        return this.mobileDeviceLibrary.AFCDirectoryOpen(afcConnection, path, afcDirectory);
    };

    MobileDevice.prototype.afcDirectoryRead = function (afcConnection, afcDirectory, name) {
        return this.mobileDeviceLibrary.AFCDirectoryRead(afcConnection, afcDirectory, name);
    };

    MobileDevice.prototype.afcDirectoryClose = function (afcConnection, afcDirectory) {
        return this.mobileDeviceLibrary.AFCDirectoryClose(afcConnection, afcDirectory);
    };

    MobileDevice.prototype.isDataReceivingCompleted = function (reply) {
        if (reply.indexOf("Error") >= 0) {
            var openingTag = "<string>";
            var closingTag = "</string>";
            if (reply.indexOf(closingTag) >= 0) {
                this.$errors.fail(reply.substring(reply.indexOf(openingTag) + openingTag.length, reply.indexOf(closingTag)));
            }
        }

        return reply.indexOf("Status") >= 0 && reply.indexOf("Complete") >= 0 && reply.indexOf("PercentComplete") < 0;
    };
    return MobileDevice;
})();
exports.MobileDevice = MobileDevice;
$injector.register("mobileDevice", MobileDevice);

var WinSocket = (function () {
    function WinSocket(service, $logger, $mobileDevice, $errors) {
        this.service = service;
        this.$logger = $logger;
        this.$mobileDevice = $mobileDevice;
        this.$errors = $errors;
        this.winSocketLibrary = null;
        this.winSocketLibrary = IOSCore.getWinSocketLibrary();
    }
    WinSocket.prototype.read = function (bytes) {
        var _this = this;
        var data = new Buffer(bytes);
        var result;
        helpers.block(function () {
            result = _this.winSocketLibrary.recv(_this.service, data, bytes, 0);
        });
        if (result < 0) {
            this.$errors.fail("Error receiving data: %s", result);
        } else if (result === 0) {
            return null;
        }

        return data;
    };

    WinSocket.prototype.readSystemLog = function (printData) {
        var data = this.read(WinSocket.BYTES_TO_READ);
        while (data) {
            printData(data);
            data = this.read(WinSocket.BYTES_TO_READ);
        }
        this.close();
    };

    WinSocket.prototype.receiveMessage = function () {
        var _this = this;
        return (function () {
            var reply = _this.receiveMessageHelper();
            if (!_this.$mobileDevice.isDataReceivingCompleted(reply)) {
                reply = _this.receiveMessageHelper();
            }
        }).future()();
    };

    WinSocket.prototype.receiveMessageHelper = function () {
        var data = this.read(4);
        var reply = "";

        if (data !== null && data.length === 4) {
            var l = bufferpack.unpack(">i", data)[0];
            var left = l;
            while (left > 0) {
                var r = this.read(left);
                if (r === null) {
                    this.$errors.fail("Unable to read reply");
                }
                reply += r;
                left -= r.length;
            }
        }

        var result = reply.toString();
        this.$errors.verifyHeap("receiveMessage");
        return result;
    };

    WinSocket.prototype.sendMessage = function (data) {
        var payload = plistlib.toString(this.createPlist(data));
        var message = bufferpack.pack(">i", [payload.length]) + payload;
        var writtenBytes = this.winSocketLibrary.send(this.service, message, message.length, 0);
        this.$logger.debug("WinSocket-> sending message: '%s', written bytes: '%s'", message, writtenBytes);

        this.$errors.verifyHeap("sendMessage");
    };

    WinSocket.prototype.createPlist = function (data) {
        var keys = _.keys(data);
        var values = _.values(data);
        var plist = { type: "dict", value: {} };

        for (var i = 0; i < keys.length; i++) {
            var type = values[i] instanceof Object ? "dict" : "string";
            var value = type === "dict" ? {} : values[i];

            plist.value[keys[i]] = { type: type, value: value };
        }

        this.$logger.trace("created plist: '%s'" + plist);

        return plist;
    };

    WinSocket.prototype.close = function () {
        this.winSocketLibrary.closesocket(this.service);
        this.$errors.verifyHeap("socket close");
    };
    WinSocket.BYTES_TO_READ = 1024;
    return WinSocket;
})();

var PosixSocket = (function () {
    function PosixSocket(service, $coreFoundation, $mobileDevice, $logger, $errors) {
        this.$coreFoundation = $coreFoundation;
        this.$mobileDevice = $mobileDevice;
        this.$logger = $logger;
        this.$errors = $errors;
        this.socket = null;
        this.socket = new net.Socket({ fd: service });
    }
    PosixSocket.prototype.receiveMessage = function () {
        var _this = this;
        var result = new Future();

        this.socket.on("data", function (data) {
            _this.$logger.debug("PosixSocket receiving: '%s'", data.toString());
            _this.$errors.verifyHeap("receiveMessage");
            var reply = data.toString();

            try  {
                if (_this.$mobileDevice.isDataReceivingCompleted(reply)) {
                    result.return();
                }
            } catch (e) {
                result.throw(e);
            }
        }).on("error", function (error) {
            result.throw(error);
        });

        return result;
    };

    PosixSocket.prototype.readSystemLog = function (action) {
        var _this = this;
        this.socket.on("data", function (data) {
            action(data);
        }).on("end", function () {
            _this.close();
            _this.$errors.verifyHeap("readSystemLog");
        }).on("error", function (error) {
            _this.$errors.fail(error);
        });
    };

    PosixSocket.prototype.sendMessage = function (message, format) {
        var data = this.$coreFoundation.dictToPlistEncoding(message, format);
        var payload = bufferpack.pack(">i", [data.length]);

        this.$logger.trace("PlistService sending: ");
        this.$logger.trace(data.toString());

        this.socket.write(payload);
        this.socket.write(data);

        this.$errors.verifyHeap("sendMessage");
    };

    PosixSocket.prototype.close = function () {
        this.socket.destroy();
        this.$errors.verifyHeap("socket close");
    };
    return PosixSocket;
})();

var PlistService = (function () {
    function PlistService(service, format, $injector) {
        this.service = service;
        this.format = format;
        this.$injector = $injector;
        this.socket = null;
        if (hostInfo.isWindows()) {
            this.socket = this.$injector.resolve(WinSocket, { service: this.service });
        } else if (hostInfo.isDarwin()) {
            this.socket = this.$injector.resolve(PosixSocket, { service: this.service });
        }
    }
    PlistService.prototype.receiveMessage = function () {
        return this.socket.receiveMessage();
    };

    PlistService.prototype.readSystemLog = function (action) {
        this.socket.readSystemLog(action);
    };

    PlistService.prototype.sendMessage = function (message) {
        this.socket.sendMessage(message, this.format);
    };

    PlistService.prototype.close = function () {
        this.socket.close();
    };
    return PlistService;
})();
exports.PlistService = PlistService;
