var ref = require("ref");

var iOSProxyServices = require("./ios-proxy-services");

var MobileHelper = require("./../mobile-helper");

var IOSDevice = (function () {
    function IOSDevice(devicePointer, $coreFoundation, $mobileDevice, $errors, $logger, $injector) {
        this.devicePointer = devicePointer;
        this.$coreFoundation = $coreFoundation;
        this.$mobileDevice = $mobileDevice;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$injector = $injector;
        this.identifier = null;
        this.voidPtr = ref.refType(ref.types.void);
    }
    IOSDevice.prototype.getPlatform = function () {
        return MobileHelper.DevicePlatforms[0 /* iOS */];
    };

    IOSDevice.prototype.getIdentifier = function () {
        if (this.identifier == null) {
            this.identifier = this.$coreFoundation.convertCFStringToCString(this.$mobileDevice.deviceCopyDeviceIdentifier(this.devicePointer));
        }

        return this.identifier;
    };

    IOSDevice.prototype.getDisplayName = function () {
        return this.getValue("ProductType");
    };

    IOSDevice.prototype.getModel = function () {
        return this.getValue("ProductType");
    };

    IOSDevice.prototype.getVersion = function () {
        return this.getValue("ProductVersion");
    };

    IOSDevice.prototype.getVendor = function () {
        return "Apple";
    };

    IOSDevice.prototype.getInstalledApplications = function () {
        throw new Error("Not implemented");
    };

    IOSDevice.prototype.getValue = function (value) {
        this.connect();
        this.startSession();
        try  {
            var cfValue = this.$coreFoundation.createCFString(value);
            return this.$coreFoundation.convertCFStringToCString(this.$mobileDevice.deviceCopyValue(this.devicePointer, null, cfValue));
        } finally {
            this.stopSession();
            this.disconnect();
        }
    };

    IOSDevice.prototype.validateResult = function (result, error) {
        if (result != 0) {
            this.$errors.fail(error);
        }
    };

    IOSDevice.prototype.isPaired = function () {
        return this.$mobileDevice.deviceIsPaired(this.devicePointer) != 0;
    };

    IOSDevice.prototype.pair = function () {
        var result = this.$mobileDevice.devicePair(this.devicePointer);
        this.validateResult(result, "If your phone is locked with a passcode, unlock then reconnect it");
        return result;
    };

    IOSDevice.prototype.validatePairing = function () {
        var result = this.$mobileDevice.deviceValidatePairing(this.devicePointer);
        this.validateResult(result, "Unable to validate pairing");
        return result;
    };

    IOSDevice.prototype.connect = function () {
        var result = this.$mobileDevice.deviceConnect(this.devicePointer);
        this.validateResult(result, "Unable to connect to device");

        if (!this.isPaired()) {
            this.pair();
        }

        return this.validatePairing();
    };

    IOSDevice.prototype.disconnect = function () {
        var result = this.$mobileDevice.deviceDisconnect(this.devicePointer);
        this.validateResult(result, "Unable to disconnect from device");
    };

    IOSDevice.prototype.startSession = function () {
        var result = this.$mobileDevice.deviceStartSession(this.devicePointer);
        this.validateResult(result, "Unable to start session");
    };

    IOSDevice.prototype.stopSession = function () {
        var result = this.$mobileDevice.deviceStopSession(this.devicePointer);
        this.validateResult(result, "Unable to stop session");
    };

    IOSDevice.prototype.startService = function (serviceName) {
        this.connect();
        try  {
            this.startSession();
            try  {
                var socket = ref.alloc("int");
                var result = this.$mobileDevice.deviceStartService(this.devicePointer, this.$coreFoundation.createCFString(serviceName), socket);
                this.validateResult(result, "Unable to start service");
                return ref.deref(socket);
            } finally {
                this.stopSession();
            }
        } finally {
            this.disconnect();
        }
    };

    IOSDevice.prototype.deploy = function (packageFile, packageName) {
        var _this = this;
        return (function () {
            var installationProxy = _this.$injector.resolve(iOSProxyServices.InstallationProxyClient, { device: _this });
            installationProxy.deployApplication(packageFile).wait();
            installationProxy.closeSocket();
        }).future()();
    };

    IOSDevice.prototype.sync = function (localToDevicePaths, appIdentifier, projectType, options) {
        var _this = this;
        if (typeof options === "undefined") { options = {}; }
        return (function () {
            var houseArrestClient = _this.$injector.resolve(iOSProxyServices.HouseArrestClient, { device: _this });
            var afcClientForAppDocuments = houseArrestClient.getAfcClientForAppDocuments(appIdentifier.appIdentifier);
            afcClientForAppDocuments.transferCollection(localToDevicePaths).wait();
            houseArrestClient.closeSocket();

            if (!options.skipRefresh) {
                var afcClientForContainer = houseArrestClient.getAfcClientForAppContainer(appIdentifier.appIdentifier);
                afcClientForContainer.deleteFile("/Library/Preferences/ServerInfo.plist");
                houseArrestClient.closeSocket();

                var notificationProxyClient = _this.$injector.resolve(iOSProxyServices.NotificationProxyClient, { device: _this });
                notificationProxyClient.postNotification("com.telerik.app.refreshWebView");
                notificationProxyClient.closeSocket();
            }

            _this.$logger.out("Successfully synced device with identifier '%s'", _this.getIdentifier());
        }).future()();
    };

    IOSDevice.prototype.openDeviceLogStream = function () {
        var iOSSystemLog = this.$injector.resolve(iOSProxyServices.IOSSyslog, { device: this });
        iOSSystemLog.read();
    };
    return IOSDevice;
})();
exports.IOSDevice = IOSDevice;

$injector.register("iOSDevice", IOSDevice);
