var ref = require("ref");

var path = require("path");
var iOSCore = require("./ios-core");

var helpers = require("./../../helpers");

var MobileServices = (function () {
    function MobileServices() {
    }
    MobileServices.APPLE_FILE_CONNECTION = "com.apple.afc";
    MobileServices.INSTALLATION_PROXY = "com.apple.mobile.installation_proxy";
    MobileServices.HOUSE_ARREST = "com.apple.mobile.house_arrest";
    MobileServices.NOTIFICATION_PROXY = "com.apple.mobile.notification_proxy";
    MobileServices.SYSLOG = "com.apple.syslog_relay";
    return MobileServices;
})();

var AfcFile = (function () {
    function AfcFile(path, mode, afcConnection, $mobileDevice, $errors) {
        this.afcConnection = afcConnection;
        this.$mobileDevice = $mobileDevice;
        this.$errors = $errors;
        this.open = false;
        var modeValue = 0;
        if (mode.indexOf("r") > -1) {
            modeValue = 0x1;
        }
        if (mode.indexOf("w") > -1) {
            modeValue = 0x2;
        }
        var afcFileRef = ref.alloc(ref.types.uint64);
        this.open = false;

        var result = this.$mobileDevice.afcFileRefOpen(afcConnection, path, modeValue, afcFileRef);
        if (result !== 0) {
            this.$errors.fail("Unable to open file reference: '%s' with path '%s", result, path);
        }

        this.afcFile = ref.deref(afcFileRef);
        if (this.afcFile === 0) {
            this.$errors.fail("Invalid file reference");
        }

        this.open = true;
    }
    AfcFile.prototype.write = function (buffer, byteLength) {
        var result = this.$mobileDevice.afcFileRefWrite(this.afcConnection, this.afcFile, buffer, byteLength);
        if (result !== 0) {
            this.$errors.fail("Unable to write to file: '%s'. Result is: '%s'", this.afcFile, result);
        }

        return true;
    };

    AfcFile.prototype.close = function () {
        if (this.open) {
            var result = this.$mobileDevice.afcFileRefClose(this.afcConnection, this.afcFile);
            if (result !== 0) {
                this.$errors.fail("Unable to close afc file connection: '%s'. Result is: '%s'", this.afcFile, result);
            }
            this.open = false;
        }
    };

    Object.defineProperty(AfcFile.prototype, "writable", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    return AfcFile;
})();
exports.AfcFile = AfcFile;

var AfcClient = (function () {
    function AfcClient(service, $mobileDevice, $fs, $errors, $logger, $injector) {
        this.service = service;
        this.$mobileDevice = $mobileDevice;
        this.$fs = $fs;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$injector = $injector;
        this.afcConnection = null;
        var afcConnection = ref.alloc(ref.refType(ref.types.void));
        var result = $mobileDevice.afcConnectionOpen(this.service, 0, afcConnection);
        if (result !== 0) {
            $errors.fail("Unable to open apple file connection: %s", result);
        }

        this.afcConnection = ref.deref(afcConnection);
    }
    AfcClient.prototype.open = function (path, mode) {
        return this.$injector.resolve(AfcFile, { path: path, mode: mode, afcConnection: this.afcConnection });
    };

    AfcClient.prototype.mkdir = function (path) {
        var result = this.$mobileDevice.afcDirectoryCreate(this.afcConnection, path);
        if (result !== 0) {
            this.$errors.fail("Unable to make directory: %s. Result is %s", path, result);
        }
    };

    AfcClient.prototype.listDir = function (path) {
        var afcDirectoryRef = ref.alloc(ref.refType(ref.types.void));
        var result = this.$mobileDevice.afcDirectoryOpen(this.afcConnection, path, afcDirectoryRef);
        if (result !== 0) {
            this.$errors.fail("Unable to open AFC directory: '%s' %s ", path, result);
        }

        var afcDirectoryValue = ref.deref(afcDirectoryRef);
        var name = ref.alloc(ref.refType(ref.types.char));

        while (this.$mobileDevice.afcDirectoryRead(this.afcConnection, afcDirectoryValue, name) === 0) {
            var value = ref.deref(name);
            if (ref.address(value) === 0) {
                break;
            }
            var filePath = ref.readCString(value, 0);
            if (filePath !== "." && filePath !== "..") {
                console.log(filePath);
            }
        }

        this.$mobileDevice.afcDirectoryClose(this.afcConnection, afcDirectoryValue);
    };

    AfcClient.prototype.transferPackage = function (localFilePath, devicePath) {
        var _this = this;
        return (function () {
            _this.transfer(localFilePath, devicePath).wait();
        }).future()();
    };

    AfcClient.prototype.transferCollection = function (localToDevicePaths) {
        var _this = this;
        return (function () {
            localToDevicePaths.forEach(function (localToDevicePathData) {
                _this.transfer(localToDevicePathData.getLocalPath(), path.join("/Documents", localToDevicePathData.getRelativeToProjectBasePath())).wait();
            });
        }).future()();
    };

    AfcClient.prototype.deleteFile = function (devicePath) {
        var removeResult = this.$mobileDevice.afcRemovePath(this.afcConnection, devicePath);
        this.$logger.trace("Removing device file '%s', result: %s", devicePath, removeResult.toString());
    };

    AfcClient.prototype.transfer = function (localFilePath, devicePath) {
        var _this = this;
        return (function () {
            _this.ensureDevicePathExist(path.dirname(devicePath));
            var reader = _this.$fs.createReadStream(localFilePath);
            devicePath = helpers.fromWindowsRelativePathToUnix(devicePath);

            _this.deleteFile(devicePath);

            var target = _this.open(devicePath, "w");
            var localFilePathSize = _this.$fs.getFileSize(localFilePath).wait();

            reader.on("data", function (data) {
                target.write(data, data.length);
                _this.$logger.trace("transfer-> localFilePath: '%s', devicePath: '%s', localFilePathSize: '%s', transferred bytes: '%s'", localFilePath, devicePath, localFilePathSize.toString(), data.length.toString());
            }).on("error", function (error) {
                _this.$errors.fail(error);
            }).on("end", function () {
                return target.close();
            });

            _this.$fs.futureFromEvent(reader, "close").wait();
        }).future()();
    };

    AfcClient.prototype.ensureDevicePathExist = function (deviceDirPath) {
        var _this = this;
        var filePathParts = deviceDirPath.split(path.sep);
        var currentDevicePath = "";

        filePathParts.forEach(function (filePathPart) {
            if (filePathPart !== "") {
                currentDevicePath = helpers.fromWindowsRelativePathToUnix(path.join(currentDevicePath, filePathPart));
                _this.mkdir(currentDevicePath);
            }
        });
    };
    return AfcClient;
})();
exports.AfcClient = AfcClient;

var InstallationProxyClient = (function () {
    function InstallationProxyClient(device, $logger, $injector) {
        this.device = device;
        this.$logger = $logger;
        this.$injector = $injector;
        this.plistService = null;
    }
    InstallationProxyClient.prototype.deployApplication = function (packageFile) {
        var _this = this;
        return (function () {
            var service = _this.device.startService(MobileServices.APPLE_FILE_CONNECTION);
            var afcClient = _this.$injector.resolve(AfcClient, { service: service });
            var devicePath = path.join("PublicStaging", path.basename(packageFile));

            afcClient.transferPackage(packageFile, devicePath).wait();
            _this.plistService = _this.$injector.resolve(iOSCore.PlistService, { service: _this.device.startService(MobileServices.INSTALLATION_PROXY), format: iOSCore.CoreTypes.kCFPropertyListBinaryFormat_v1_0 });

            _this.plistService.sendMessage({
                Command: "Install",
                "PackagePath": helpers.fromWindowsRelativePathToUnix(devicePath)
            });
            _this.plistService.receiveMessage().wait();
            _this.$logger.info("Successfully deployed on device %s", _this.device.getIdentifier());
        }).future()();
    };

    InstallationProxyClient.prototype.closeSocket = function () {
        return this.plistService.close();
    };
    return InstallationProxyClient;
})();
exports.InstallationProxyClient = InstallationProxyClient;
$injector.register("installationProxyClient", InstallationProxyClient);

var NotificationProxyClient = (function () {
    function NotificationProxyClient(device, $injector) {
        this.device = device;
        this.$injector = $injector;
        this.plistService = null;
    }
    NotificationProxyClient.prototype.postNotification = function (notificationName) {
        this.plistService = this.$injector.resolve(iOSCore.PlistService, { service: this.device.startService(MobileServices.NOTIFICATION_PROXY), format: iOSCore.CoreTypes.kCFPropertyListBinaryFormat_v1_0 });

        var result = this.plistService.sendMessage({
            "Command": "PostNotification",
            "Name": notificationName,
            "ClientOptions": ""
        });

        return result;
    };

    NotificationProxyClient.prototype.closeSocket = function () {
        this.plistService.close();
    };
    return NotificationProxyClient;
})();
exports.NotificationProxyClient = NotificationProxyClient;

var HouseArrestClient = (function () {
    function HouseArrestClient(device, $injector) {
        this.device = device;
        this.$injector = $injector;
        this.plistService = null;
    }
    HouseArrestClient.prototype.getAfcClientCore = function (command, applicationIdentifier) {
        var service = this.device.startService(MobileServices.HOUSE_ARREST);
        this.plistService = this.$injector.resolve(iOSCore.PlistService, { service: service, format: iOSCore.CoreTypes.kCFPropertyListXMLFormat_v1_0 });

        this.plistService.sendMessage({
            "Command": command,
            "Identifier": applicationIdentifier
        });

        this.plistService.receiveMessage().wait();

        return this.$injector.resolve(AfcClient, { service: service });
    };

    HouseArrestClient.prototype.getAfcClientForAppDocuments = function (applicationIdentifier) {
        return this.getAfcClientCore("VendDocuments", applicationIdentifier);
    };

    HouseArrestClient.prototype.getAfcClientForAppContainer = function (applicationIdentifier) {
        return this.getAfcClientCore("VendContainer", applicationIdentifier);
    };

    HouseArrestClient.prototype.closeSocket = function () {
        this.plistService.close();
    };
    return HouseArrestClient;
})();
exports.HouseArrestClient = HouseArrestClient;

var IOSSyslog = (function () {
    function IOSSyslog(device, $logger, $injector) {
        this.device = device;
        this.$logger = $logger;
        this.$injector = $injector;
        this.matchRegex = new RegExp(".*?((Cordova.{3}|AppBuilder)\\[\\d+\\] <Warning>: )", 'i');
        this.plistService = this.$injector.resolve(iOSCore.PlistService, { service: this.device.startService(MobileServices.SYSLOG), format: undefined });
    }
    IOSSyslog.prototype.read = function () {
        var _this = this;
        var printData = function (data) {
            var output = ref.readCString(data, 0);
            if (_this.matchRegex.test(output)) {
                _this.$logger.out(output);
            }
        };

        this.plistService.readSystemLog(printData);
    };
    return IOSSyslog;
})();
exports.IOSSyslog = IOSSyslog;
