var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var ref = require("ref");
var util = require("util");
var os = require("os");
var path = require("path");
var IOSDevice = require("./../ios/ios-device");
var AndroidDevice = require("./../android/android-device");
var CoreTypes = require("./../ios/ios-core");
var Signal = require("./../../events/signal");
var Future = require("fibers/future");

var hostInfo = require("../../host-info");
var options = require("./../../options");

var DeviceDiscovery = (function () {
    function DeviceDiscovery() {
        this.devices = {};
        this.deviceFound = null;
        this.deviceLost = null;
        this.deviceFound = new Signal.Signal();
        this.deviceLost = new Signal.Signal();
    }
    DeviceDiscovery.prototype.addDevice = function (device) {
        this.devices[device.getIdentifier()] = device;
        this.raiseOnDeviceFound(device);
    };

    DeviceDiscovery.prototype.removeDevice = function (deviceIdentifier) {
        var device = this.devices[deviceIdentifier];
        if (!device) {
            return;
        }
        delete this.devices[deviceIdentifier];
        this.raiseOnDeviceLost(device);
    };

    DeviceDiscovery.prototype.startLookingForDevices = function () {
        return undefined;
    };

    DeviceDiscovery.prototype.raiseOnDeviceFound = function (device) {
        this.deviceFound.dispatch(device);
    };

    DeviceDiscovery.prototype.raiseOnDeviceLost = function (device) {
        this.deviceLost.dispatch(device);
    };
    return DeviceDiscovery;
})();
exports.DeviceDiscovery = DeviceDiscovery;
$injector.register("deviceDiscovery", DeviceDiscovery);

var IOSDeviceDiscovery = (function (_super) {
    __extends(IOSDeviceDiscovery, _super);
    function IOSDeviceDiscovery($coreFoundation, $mobileDevice, $errors, $injector) {
        _super.call(this);
        this.$coreFoundation = $coreFoundation;
        this.$mobileDevice = $mobileDevice;
        this.$errors = $errors;
        this.$injector = $injector;
        this.timerCallbackPtr = null;
        this.notificationCallbackPtr = null;
        this.timerCallbackPtr = CoreTypes.CoreTypes.cf_run_loop_timer_callback.toPointer(IOSDeviceDiscovery.timerCallback);
        this.notificationCallbackPtr = CoreTypes.CoreTypes.am_device_notification_callback.toPointer(IOSDeviceDiscovery.deviceNotificationCallback);
    }
    IOSDeviceDiscovery.prototype.startLookingForDevices = function () {
        var _this = this;
        return (function () {
            _this.subscribeForNotifications();
            var defaultTimeoutInSeconds = options.timeout ? parseInt(options.timeout, 10) / 1000 : 1;
            _this.startRunLoopWithTimer(defaultTimeoutInSeconds);
        }).future()();
    };

    IOSDeviceDiscovery.deviceNotificationCallback = function (devicePointer, user) {
        var iOSDeviceDiscovery = $injector.resolve("iOSDeviceDiscovery");
        var deviceInfo = ref.deref(devicePointer);

        if (deviceInfo.msg === IOSDeviceDiscovery.ADNCI_MSG_CONNECTED) {
            iOSDeviceDiscovery.createAndAddDevice(deviceInfo.dev);
        } else if (deviceInfo.msg === IOSDeviceDiscovery.ADNCI_MSG_DISCONNECTED) {
            var deviceIdentifier = iOSDeviceDiscovery.$coreFoundation.convertCFStringToCString(iOSDeviceDiscovery.$mobileDevice.deviceCopyDeviceIdentifier(deviceInfo.dev));
            iOSDeviceDiscovery.removeDevice(deviceIdentifier);
        }
    };

    IOSDeviceDiscovery.timerCallback = function () {
        var iOSDeviceDiscovery = $injector.resolve("iOSDeviceDiscovery");
        iOSDeviceDiscovery.$coreFoundation.runLoopStop(iOSDeviceDiscovery.$coreFoundation.runLoopGetCurrent());
    };

    IOSDeviceDiscovery.prototype.validateResult = function (result, error) {
        if (result != 0) {
            this.$errors.fail(error);
        }
    };

    IOSDeviceDiscovery.prototype.subscribeForNotifications = function () {
        var notifyFunction = ref.alloc(CoreTypes.CoreTypes.amDeviceNotificationRef);

        var result = this.$mobileDevice.deviceNotificationSubscribe(this.notificationCallbackPtr, 0, 0, 0, notifyFunction);
        var error = IOSDeviceDiscovery.APPLE_SERVICE_NOT_STARTED_ERROR_CODE ? "Cannot run and complete operations on iOS devices because Apple Mobile Device Service is not started. Verify that iTunes is installed and running on your system." : "Unable to subscribe for notifications";
        this.validateResult(result, error);
        this.$errors.verifyHeap("subscribeForNotifications");
    };

    IOSDeviceDiscovery.prototype.startRunLoopWithTimer = function (timeout) {
        var kCFRunLoopDefaultMode = this.$coreFoundation.kCFRunLoopDefaultMode();
        var timer = null;

        if (timeout > 0) {
            var currentTime = this.$coreFoundation.absoluteTimeGetCurrent() + timeout;
            timer = this.$coreFoundation.runLoopTimerCreate(null, currentTime, 0, 0, 0, this.timerCallbackPtr, null);
            this.$coreFoundation.runLoopAddTimer(this.$coreFoundation.runLoopGetCurrent(), timer, kCFRunLoopDefaultMode);
        }

        this.$coreFoundation.runLoopRun();

        if (timeout > 0) {
            this.$coreFoundation.runLoopRemoveTimer(this.$coreFoundation.runLoopGetCurrent(), timer, kCFRunLoopDefaultMode);
        }

        this.$errors.verifyHeap("startRunLoopWithTimer");
    };

    IOSDeviceDiscovery.prototype.createAndAddDevice = function (devicePointer) {
        var device = this.$injector.resolve(IOSDevice.IOSDevice, { devicePointer: devicePointer });
        this.addDevice(device);
    };
    IOSDeviceDiscovery.ADNCI_MSG_CONNECTED = 1;
    IOSDeviceDiscovery.ADNCI_MSG_DISCONNECTED = 2;
    IOSDeviceDiscovery.APPLE_SERVICE_NOT_STARTED_ERROR_CODE = 0xE8000063;
    return IOSDeviceDiscovery;
})(DeviceDiscovery);

var IOSDeviceDiscoveryStub = (function (_super) {
    __extends(IOSDeviceDiscoveryStub, _super);
    function IOSDeviceDiscoveryStub($logger, error) {
        _super.call(this);
        this.$logger = $logger;
        this.error = error;
    }
    IOSDeviceDiscoveryStub.prototype.startLookingForDevices = function () {
        this.$logger.warn(this.error);
        return Future.fromResult();
    };
    return IOSDeviceDiscoveryStub;
})(DeviceDiscovery);

var ITunesValidator = (function () {
    function ITunesValidator($fs) {
        this.$fs = $fs;
    }
    ITunesValidator.prototype.getError = function () {
        var _this = this;
        return (function () {
            if (hostInfo.isWindows64()) {
                if (process.arch === "x64") {
                    return "To be able to run operations on connected iOS devices, install the 32-bit version of Node.js.";
                }
            }

            var coreFoundationDir = "";
            var mobileDeviceDir = "";

            if (hostInfo.isWindows()) {
                var commonProgramFiles = hostInfo.isWindows64() ? process.env["CommonProgramFiles(x86)"] : process.env.CommonProgramFiles;
                coreFoundationDir = path.join(commonProgramFiles, "Apple", "Apple Application Support");
                mobileDeviceDir = path.join(commonProgramFiles, "Apple", "Mobile Device Support");
            } else if (hostInfo.isDarwin()) {
                coreFoundationDir = "/System/Library/Frameworks/CoreFoundation.framework/CoreFoundation";
                mobileDeviceDir = "/System/Library/PrivateFrameworks/MobileDevice.framework/MobileDevice";
            }

            var existsCoreFoundation = _this.$fs.exists(coreFoundationDir).wait();
            var existsMobileDevice = _this.$fs.exists(mobileDeviceDir).wait();

            if (!existsCoreFoundation || !existsMobileDevice) {
                return ITunesValidator.NOT_INSTALLED_iTUNES_ERROR_MESSAGE;
            }

            return null;
        }).future()();
    };
    ITunesValidator.NOT_INSTALLED_iTUNES_ERROR_MESSAGE = "iTunes is not installed. Install it on your system and run this command again.";
    return ITunesValidator;
})();

$injector.register("iOSDeviceDiscovery", function ($errors, $logger, $fs, $injector) {
    var iTunesValidator = new ITunesValidator($fs);
    var error = iTunesValidator.getError().wait();
    var result = null;

    if (error) {
        result = new IOSDeviceDiscoveryStub($logger, error);
    } else {
        result = $injector.resolve(IOSDeviceDiscovery);
    }

    return result;
});

var AndroidDeviceDiscovery = (function (_super) {
    __extends(AndroidDeviceDiscovery, _super);
    function AndroidDeviceDiscovery($childProcess, $injector, $staticConfig) {
        _super.call(this);
        this.$childProcess = $childProcess;
        this.$injector = $injector;
        this.$staticConfig = $staticConfig;
    }
    Object.defineProperty(AndroidDeviceDiscovery.prototype, "Adb", {
        get: function () {
            if (!AndroidDeviceDiscovery.adb) {
                AndroidDeviceDiscovery.adb = this.$staticConfig.adbFilePath;
            }

            return AndroidDeviceDiscovery.adb;
        },
        enumerable: true,
        configurable: true
    });

    AndroidDeviceDiscovery.prototype.createAndAddDevice = function (deviceIdentifier) {
        var _this = this;
        return (function () {
            var device = _this.$injector.resolve(AndroidDevice.AndroidDevice, {
                identifier: deviceIdentifier, adb: _this.Adb
            });
            _this.addDevice(device);
        }).future()();
    };

    AndroidDeviceDiscovery.prototype.startLookingForDevices = function () {
        var _this = this;
        return (function () {
            _this.ensureAdbServerStarted().wait();

            var requestAllDevicesCommand = util.format("%s devices", _this.Adb);
            var result = _this.$childProcess.exec(requestAllDevicesCommand).wait();

            var devices = result.toString().split(os.EOL).slice(1).filter(function (element) {
                return element && !element.isEmpty();
            }).map(function (element) {
                var parts = element.split("\t");
                var identifier = parts[0];
                var state = parts[1];
                if (state === "device") {
                    _this.createAndAddDevice(identifier).wait();
                }
            });
        }).future()();
    };

    AndroidDeviceDiscovery.prototype.ensureAdbServerStarted = function () {
        var startAdbServerCommand = util.format("%s start-server", this.Adb);
        return this.$childProcess.exec(startAdbServerCommand);
    };
    return AndroidDeviceDiscovery;
})(DeviceDiscovery);
exports.AndroidDeviceDiscovery = AndroidDeviceDiscovery;
$injector.register("androidDeviceDiscovery", AndroidDeviceDiscovery);
