var jaroWinklerDistance = require("../vendor/jaro-winkler_distance");
var helpers = require("./../helpers");
var util = require("util");

var CommandsService = (function () {
    function CommandsService($errors, $logger, $injector, $staticConfig, $hooksService) {
        this.$errors = $errors;
        this.$logger = $logger;
        this.$injector = $injector;
        this.$staticConfig = $staticConfig;
        this.$hooksService = $hooksService;
    }
    CommandsService.prototype.allCommands = function (includeDev) {
        var commands = this.$injector.getRegisteredCommandsNames(includeDev);
        return _.reject(commands, function (command) {
            return _.contains(command, '|');
        });
    };

    CommandsService.prototype.executeCommandUnchecked = function (commandName, commandArguments) {
        var _this = this;
        return (function () {
            var command = _this.$injector.resolveCommand(commandName);
            if (command) {
                if (!command.disableAnalytics) {
                    var analyticsService = _this.$injector.resolve("analyticsService");
                    analyticsService.checkConsent(commandName).wait();
                    analyticsService.trackFeature(commandName).wait();
                }
                if (command.enableHooks === undefined || command.enableHooks === true) {
                    var childrenCommandsNames = _.map(_this.$injector.getChildrenCommandsNames(commandName), function (command) {
                        return command.replace("*", "");
                    });
                    if (_.contains(childrenCommandsNames, commandArguments[0])) {
                        commandName = util.format("%s-%s", commandName, commandArguments[0]);
                    }

                    _this.$hooksService.initialize(commandName);
                    _this.$hooksService.executeBeforeHooks().wait();
                    command.execute(commandArguments).wait();
                    _this.$hooksService.executeAfterHooks().wait();
                } else {
                    command.execute(commandArguments).wait();
                }
                return true;
            }
            return false;
        }).future()();
    };

    CommandsService.prototype.executeCommand = function (commandName, commandArguments) {
        var _this = this;
        return this.$errors.beginCommand(function () {
            return _this.executeCommandUnchecked(commandName, commandArguments);
        }, function () {
            return _this.executeCommandUnchecked("help", [_this.beautifyCommandName(commandName)]);
        });
    };

    CommandsService.prototype.tryExecuteCommand = function (commandName, commandArguments) {
        var _this = this;
        return (function () {
            if (!_this.executeCommand(commandName, commandArguments).wait()) {
                _this.$logger.fatal("Unknown command '%s'. Use '%s help' for help.", helpers.stringReplaceAll(commandName, "|", " "), _this.$staticConfig.CLIENT_NAME);
                _this.tryMatchCommand(commandName);
            }
        }).future()();
    };

    CommandsService.prototype.tryMatchCommand = function (commandName) {
        var _this = this;
        var allCommands = this.allCommands(false);
        var similarCommands = [];
        _.each(allCommands, function (command) {
            if (!_this.$injector.isDefaultCommand(command)) {
                command = helpers.stringReplaceAll(command, "|", " ");
                var distance = jaroWinklerDistance(commandName, command);
                if (commandName.length > 3 && command.indexOf(commandName) != -1) {
                    similarCommands.push({ rating: 1, name: command });
                } else if (distance >= 0.65) {
                    similarCommands.push({ rating: distance, name: command });
                }
            }
        });

        similarCommands = _.sortBy(similarCommands, function (command) {
            return -command.rating;
        }).slice(0, 5);

        if (similarCommands.length > 0) {
            var message = ["Did you mean?"];
            _.each(similarCommands, function (command) {
                message.push("\t" + command.name);
            });
            this.$logger.fatal(message.join("\n"));
        }
    };

    CommandsService.prototype.completeCommand = function (commandsWithPlatformArgument, platforms, getPropSchemaAction) {
        var _this = this;
        return (function () {
            var completeCallback = function (err, data) {
                if (err || !data) {
                    return;
                }

                var childrenCommands = _this.$injector.getChildrenCommandsNames(data.prev);

                if (data.words == 1) {
                    return tabtab.log(_this.allCommands(false), data);
                }

                if (data.last.startsWith("--")) {
                    return tabtab.log(Object.keys(require("./options").knownOpts), data, "--");
                }

                if (_.contains(commandsWithPlatformArgument, data.prev)) {
                    return tabtab.log(platforms, data);
                }

                if (data.words == 2 && childrenCommands) {
                    return tabtab.log(_.reject(childrenCommands, function (children) {
                        return children[0] === '*';
                    }), data);
                }

                var propSchema = getPropSchemaAction ? getPropSchemaAction() : null;

                if (propSchema) {
                    var propertyCommands = ["print", "set", "add", "del"];
                    var parseResult = /prop ([^ ]+) ([^ ]*)/.exec(data.line);
                    if (parseResult) {
                        if (_.contains(propertyCommands, parseResult[1])) {
                            var propName = parseResult[2];
                            if (propSchema[propName]) {
                                var range = propSchema[propName].range;
                                if (range) {
                                    if (!_.isArray(range)) {
                                        range = _.map(range, function (value, key) {
                                            return value.input || key;
                                        });
                                    }
                                    return tabtab.log(range, data);
                                }
                            } else {
                                return tabtab.log(Object.keys(propSchema), data);
                            }
                        }
                    }
                }

                return false;
            };

            var tabtab = require("tabtab");
            tabtab.complete(_this.$staticConfig.CLIENT_NAME.toLowerCase(), completeCallback);

            if (_this.$staticConfig.CLIENT_NAME_ALIAS) {
                tabtab.complete(_this.$staticConfig.CLIENT_NAME_ALIAS.toLowerCase(), completeCallback);
            }

            return true;
        }).future()();
    };

    CommandsService.prototype.beautifyCommandName = function (commandName) {
        if (commandName.indexOf("*") > 0) {
            return commandName.substr(0, commandName.indexOf("|"));
        }

        return commandName;
    };
    return CommandsService;
})();
exports.CommandsService = CommandsService;
$injector.register("commandsService", CommandsService);
