var path = require("path");
var util = require("util");

var Hook = (function () {
    function Hook(name, fullPath) {
        this.name = name;
        this.fullPath = fullPath;
    }
    return Hook;
})();

var HooksService = (function () {
    function HooksService($childProcess, $fs, $logger, $staticConfig, $projectHelper) {
        this.$childProcess = $childProcess;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$staticConfig = $staticConfig;
        this.$projectHelper = $projectHelper;
    }
    HooksService.prototype.initialize = function (commandName) {
        this.commandName = commandName;
        this.beforeHookName = util.format("before-%s", commandName);
        this.afterHookName = util.format("after-%s", commandName);
        this.cachedHooks = {};

        var customHooksDirectory = null;
        var relativeToLibPath = path.join(__dirname, "../../");
        var defaultHooksDirectories = [
            path.join(relativeToLibPath, HooksService.HOOKS_DIRECTORY_NAME),
            path.join(relativeToLibPath, "common", HooksService.HOOKS_DIRECTORY_NAME)
        ];

        if (this.$projectHelper.projectDir) {
            customHooksDirectory = path.join(this.$projectHelper.projectDir, HooksService.HOOKS_DIRECTORY_NAME);
        }

        this.hooksDirectories = defaultHooksDirectories.concat([customHooksDirectory]);
    };

    HooksService.prototype.executeBeforeHooks = function () {
        return this.executeHooks(this.beforeHookName);
    };

    HooksService.prototype.executeAfterHooks = function () {
        return this.executeHooks(this.afterHookName);
    };

    HooksService.prototype.executeHooks = function (hookName) {
        var _this = this;
        return (function () {
            _.each(_this.hooksDirectories, function (hooksDirectory) {
                _this.executeHook(hooksDirectory, hookName).wait();
            });
        }).future()();
    };

    HooksService.prototype.executeHook = function (directoryPath, hookName) {
        var _this = this;
        return (function () {
            var hook = _this.getHookByName(directoryPath, hookName).wait();
            if (hook) {
                var command = _this.getSheBangInterpreter(hook).wait();
                if (!command) {
                    command = hook.fullPath;
                    if (path.extname(hook.fullPath) === ".js") {
                        command = process.argv[0];
                    }
                }
                var environment = _this.prepareEnvironment(hook.fullPath);
                _this.$logger.out("Executing %s hook at location %s with environment ", hook.name, hook.fullPath, environment);
                _this.$childProcess.spawnFromEvent(command, [hook.fullPath], "close", environment).wait();
            }
        }).future()();
    };

    HooksService.prototype.getHookByName = function (directoryPath, hookName) {
        var _this = this;
        return (function () {
            var hooks = _this.getHooksInDirectory(directoryPath).wait();
            var hook = _.find(hooks, function (hook) {
                return hook.name === hookName;
            });
            return hook;
        }).future()();
    };

    HooksService.prototype.getHooksInDirectory = function (directoryPath) {
        var _this = this;
        return (function () {
            if (!_this.cachedHooks[directoryPath]) {
                var hooks = [];
                if (directoryPath && _this.$fs.exists(directoryPath).wait() && _this.$fs.getFsStats(directoryPath).wait().isDirectory()) {
                    var directoryContent = _this.$fs.readDirectory(directoryPath).wait();
                    var files = _.filter(directoryContent, function (entry) {
                        var fullPath = path.join(directoryPath, entry);
                        var isFile = _this.$fs.getFsStats(fullPath).wait().isFile();
                        var baseFilename = _this.getBaseFilename(entry);
                        return isFile && (baseFilename === _this.beforeHookName || baseFilename === _this.afterHookName);
                    });

                    hooks = _.map(files, function (file) {
                        var fullPath = path.join(directoryPath, file);
                        return new Hook(_this.getBaseFilename(file), fullPath);
                    });
                }

                _this.cachedHooks[directoryPath] = hooks;
            }

            return _this.cachedHooks[directoryPath];
        }).future()();
    };

    HooksService.prototype.prepareEnvironment = function (hookFullPath) {
        var clientName = this.$staticConfig.CLIENT_NAME.toUpperCase();

        var environment = {};
        environment[util.format("%s-COMMANDLINE", clientName)] = process.argv.join(' ');
        environment[util.format("%s-HOOK_FULL_PATH", clientName)] = hookFullPath;
        environment[util.format("%s-VERSION", clientName)] = this.$staticConfig.version;

        return {
            cwd: this.$projectHelper.projectDir,
            stdio: 'inherit',
            env: _.extend({}, process.env, environment)
        };
    };

    HooksService.prototype.getSheBangInterpreter = function (hook) {
        var _this = this;
        return (function () {
            var interpreter = null;
            var shMatch = [];
            var fileContent = _this.$fs.readText(hook.fullPath).wait();
            if (fileContent) {
                var sheBangMatch = fileContent.split('\n')[0].match(/^#!(?:\/usr\/bin\/env )?([^\r\n]+)/m);
                if (sheBangMatch) {
                    interpreter = sheBangMatch[1];
                }
                if (interpreter) {
                    shMatch = interpreter.match(/bin\/((?:ba)?sh)$/);
                }
                if (shMatch) {
                    interpreter = shMatch[1];
                }
            }

            return interpreter;
        }).future()();
    };

    HooksService.prototype.getBaseFilename = function (fileName) {
        return fileName.substr(0, fileName.length - path.extname(fileName).length);
    };
    HooksService.HOOKS_DIRECTORY_NAME = "hooks";
    return HooksService;
})();
exports.HooksService = HooksService;
$injector.register("hooksService", HooksService);
