"use strict";
var path = require("path");
var util = require("util");
var temp = require("temp");
temp.track();

var TypeScriptCompilationService = (function () {
    function TypeScriptCompilationService($childProcess, $fs, $errors, $logger, $config) {
        this.$childProcess = $childProcess;
        this.$fs = $fs;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$config = $config;
    }
    TypeScriptCompilationService.prototype.initialize = function (typeScriptFiles) {
        this.typeScriptFiles = typeScriptFiles;
    };

    TypeScriptCompilationService.prototype.compileAllFiles = function () {
        var _this = this;
        return (function () {
            if (_this.typeScriptFiles.length > 0) {
                var typeScriptCommandsFilePath = path.join(temp.mkdirSync("typeScript-compilation"), "tscommand.txt");
                var typeScriptCompilerOptions = _this.getTypeScriptCompilerOptions().wait();
                _this.$fs.writeFile(typeScriptCommandsFilePath, _this.typeScriptFiles.concat(typeScriptCompilerOptions).join(' ')).wait();

                var typeScriptModuleFilePath = require.resolve("typescript");
                var typeScriptModuleDirPath = path.dirname(typeScriptModuleFilePath);
                var typeScriptCompilerPath = path.join(typeScriptModuleDirPath, "tsc");
                var typeScriptCompilerVersion = _this.$fs.readJson(path.join(typeScriptModuleDirPath, "../", "package.json")).wait().version;

                _this.$logger.out("Compiling...".yellow);
                _.each(_this.typeScriptFiles, function (file) {
                    _this.$logger.out(util.format("### Compile ", file).cyan);
                });
                _this.$logger.out(util.format("Using tsc version ", typeScriptCompilerVersion).cyan);

                _this.runCompilation(typeScriptCompilerPath, typeScriptCommandsFilePath).wait();
            }
        }).future()();
    };

    TypeScriptCompilationService.prototype.runCompilation = function (typeScriptCompilerPath, typeScriptCommandsFilePath) {
        var _this = this;
        return (function () {
            var startTime = new Date().getTime();

            var output = _this.$childProcess.spawnFromEvent("node", [typeScriptCompilerPath, "@" + typeScriptCommandsFilePath], "close", { throwError: false }).wait();
            if (output.exitCode === 0) {
                var endTime = new Date().getTime();
                var time = (endTime - startTime) / 1000;

                _this.$logger.out(util.format("\n Success: %ss for %s typeScript files \n Done without errors.", time.toFixed(2), _this.typeScriptFiles.length).green);
            } else {
                _this.$logger.out(output.stdout);
                _this.$errors.fail("Compilation failed".red);
            }
        }).future()();
    };

    TypeScriptCompilationService.prototype.getTypeScriptCompilerOptions = function () {
        var _this = this;
        return (function () {
            var compilerOptions = [];
            var options = _this.$config.TYPESCRIPT_COMPILER_OPTIONS;

            if (options.targetVersion) {
                compilerOptions.push(options.targetVersion.toUpperCase());
            }
            if (options.module) {
                compilerOptions.push(options.module.toLowerCase());
            }

            if (options.declaration) {
                compilerOptions.push("--declaration");
            }
            if (options.noImplicitAny) {
                compilerOptions.push("--noImplicitAny");
            }
            if (options.sourceMap) {
                compilerOptions.push("--sourcemap");
            }
            if (options.removeComments) {
                compilerOptions.push("--removeComments");
            }

            if (options.out) {
                compilerOptions.push("--out", options.out);
            }
            if (options.outDir) {
                if (options.out) {
                    _this.$logger.warn("WARNING: Option out and outDir should not be used together".magenta);
                }
                compilerOptions.push("--outDir", options.outDir);
            }
            if (options.sourceRoot) {
                compilerOptions.push('--sourceRoot', options.sourceRoot);
            }
            if (options.mapRoot) {
                compilerOptions.push('--mapRoot', options.mapRoot);
            }

            return compilerOptions;
        }).future()();
    };
    return TypeScriptCompilationService;
})();
exports.TypeScriptCompilationService = TypeScriptCompilationService;
$injector.register("typeScriptCompilationService", TypeScriptCompilationService);
