var FN_NAME_AND_ARGS = /^function\s*([^\(]*)\(\s*([^\)]*)\)/m;
var FN_ARG_SPLIT = /,/;
var FN_ARG = /^\s*(_?)(\S+?)\1\s*$/;
var STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;

function annotate(fn) {
    var $inject, fnText, argDecl;

    if (typeof fn == 'function') {
        if (!($inject = fn.$inject)) {
            $inject = { args: [], name: "" };
            fnText = fn.toString().replace(STRIP_COMMENTS, '');
            argDecl = fnText.match(FN_NAME_AND_ARGS);
            $inject.name = argDecl[1];
            if (fn.length) {
                argDecl[2].split(FN_ARG_SPLIT).forEach(function (arg) {
                    arg.replace(FN_ARG, function (all, underscore, name) {
                        return $inject.args.push(name);
                    });
                });
            }
            fn.$inject = $inject;
        }
    }
    return $inject;
}

var util = require("util");
var assert = require("assert");
var Future = require("fibers/future");
var path = require("path");

var indent = "";
function trace(formatStr) {
    var args = [];
    for (var _i = 0; _i < (arguments.length - 1); _i++) {
        args[_i] = arguments[_i + 1];
    }
    formatStr = indent + formatStr;
    args.unshift(formatStr);
}

function pushIndent() {
    indent += "  ";
}

function popIndent() {
    indent = indent.slice(0, -2);
}

function forEachName(names, action) {
    if (_.isString(names)) {
        action(names);
    } else {
        names.forEach(action);
    }
}

var Yok = (function () {
    function Yok() {
        this.COMMANDS_NAMESPACE = "commands";
        this.modules = {};
        this.resolutionProgress = {};
        this.hierarchicalCommands = {};
    }
    Yok.prototype.requireCommand = function (names, file) {
        var _this = this;
        forEachName(names, function (commandName) {
            var commands = commandName.split("|");

            if (commands.length > 1) {
                if (commands[1].startsWith('*') && _this.modules[_this.createCommandName(commands[0])]) {
                    throw new Error("Default commands should be required before child commands");
                }

                var parentCommandName = commands[0];

                if (!_this.hierarchicalCommands[parentCommandName]) {
                    _this.hierarchicalCommands[parentCommandName] = [];
                }

                _this.hierarchicalCommands[parentCommandName].push(commands[1]);
            }

            if (commands.length > 1 && !_this.modules[_this.createCommandName(commands[0])]) {
                _this.require(_this.createCommandName(commands[0]), file);
            } else {
                _this.require(_this.createCommandName(commandName), file);
            }
        });
    };

    Yok.prototype.require = function (names, file) {
        var _this = this;
        forEachName(names, function (name) {
            return _this.requireOne(name, file);
        });
    };

    Yok.prototype.requireOne = function (name, file) {
        var dependency = {
            require: path.join("../", file),
            shared: true
        };
        if (!this.modules[name]) {
            this.modules[name] = dependency;
        } else {
            throw new Error(util.format("module '%s' require'd twice.", name));
        }
    };

    Yok.prototype.registerCommand = function (names, resolver) {
        var _this = this;
        forEachName(names, function (name) {
            var commands = name.split("|");
            _this.register(_this.createCommandName(name), resolver);

            if (commands.length > 1) {
                _this.createHierarchicalCommand(commands[0]);
            }
        });
    };

    Yok.prototype.getDefaultCommand = function (name) {
        var subCommands = this.hierarchicalCommands[name];
        var defaultCommand = _.find(subCommands, function (command) {
            return command.startsWith("*");
        });
        return defaultCommand;
    };

    Yok.prototype.isValidCommand = function (name) {
        var allCommands = this.getRegisteredCommandsNames(true);
        return _.contains(allCommands, name);
    };

    Yok.prototype.buildHierarchicalCommand = function (parentCommandName, childCommandName) {
        return util.format("%s|%s", parentCommandName, childCommandName);
    };

    Yok.prototype.createHierarchicalCommand = function (name) {
        var _this = this;
        var factory = function () {
            return {
                execute: function (args) {
                    return (function () {
                        var commandsService = $injector.resolve("commandsService");
                        var commandName = null;
                        var defaultCommand = _this.getDefaultCommand(name);
                        var commandArguments = [];

                        if (args.length > 0) {
                            var isValidCommand = _this.isValidCommand(_this.buildHierarchicalCommand(name, args[0]));
                            if (isValidCommand) {
                                commandName = args[0];
                                commandArguments = _.rest(args);
                            } else {
                                commandName = defaultCommand || "help";
                                commandArguments = args;
                            }
                        } else {
                            commandName = defaultCommand || "help";
                        }

                        if (commandName !== "help") {
                            commandName = _this.buildHierarchicalCommand(name, commandName);
                        }

                        commandsService.tryExecuteCommand(commandName, commandName === "help" ? [name] : commandArguments).wait();
                    }).future()();
                }
            };
        };

        $injector.registerCommand(name, factory);
    };

    Yok.prototype.isDefaultCommand = function (commandName) {
        return commandName.indexOf("*") > 0 && commandName.indexOf("|") > 0;
    };

    Yok.prototype.register = function (name, resolver, shared) {
        if (typeof shared === "undefined") { shared = true; }
        trace("registered '%s'", name);

        var dependency = this.modules[name] || {};
        dependency.shared = shared;

        if (_.isFunction(resolver)) {
            dependency.resolver = resolver;
        } else {
            dependency.instance = resolver;
        }

        this.modules[name] = dependency;
    };

    Yok.prototype.resolveCommand = function (name) {
        var command;

        var commandModuleName = this.createCommandName(name);
        if (!this.modules[commandModuleName]) {
            return null;
        }
        command = this.resolve(commandModuleName);

        return command;
    };

    Yok.prototype.resolve = function (param, ctorArguments) {
        if (_.isFunction(param)) {
            return this.resolveConstructor(param, ctorArguments);
        } else {
            assert.ok(!ctorArguments);
            return this.resolveByName(param);
        }
    };

    Object.defineProperty(Yok.prototype, "dynamicCallRegex", {
        get: function () {
            return /#{([^.]+)\.([^}]+)}/;
        },
        enumerable: true,
        configurable: true
    });

    Yok.prototype.dynamicCall = function (call, args) {
        var _this = this;
        return (function () {
            var parsed = call.match(_this.dynamicCallRegex);
            var module = _this.resolve(parsed[1]);
            var data = module[parsed[2]].apply(module, args);
            if (data && data.wait) {
                return data.wait();
            }
            return data;
        }).future()();
    };

    Yok.prototype.resolveConstructor = function (ctor, ctorArguments) {
        var _this = this;
        annotate(ctor);

        var resolvedArgs = ctor.$inject.args.map(function (paramName) {
            if (ctorArguments && ctorArguments.hasOwnProperty(paramName)) {
                return ctorArguments[paramName];
            } else {
                return _this.resolve(paramName);
            }
        });

        var name = ctor.$inject.name;
        if (name && name[0] === name[0].toUpperCase()) {
            function EmptyCtor() {
            }
            EmptyCtor.prototype = ctor.prototype;
            var obj = new EmptyCtor();

            ctor.apply(obj, resolvedArgs);
            return obj;
        } else {
            return ctor.apply(null, resolvedArgs);
        }
    };

    Yok.prototype.resolveByName = function (name) {
        if (name[0] === "$") {
            name = name.substr(1);
        }

        if (this.resolutionProgress[name]) {
            throw new Error(util.format("cyclic dependency detected on dependency '%s'", name));
        }
        this.resolutionProgress[name] = true;

        trace("resolving '%s'", name);
        pushIndent();

        try  {
            var dependency = this.resolveDependency(name);

            if (!dependency) {
                throw new Error("unable to resolve " + name);
            }

            if (!dependency.instance || !dependency.shared) {
                if (!dependency.resolver) {
                    throw new Error("no resolver registered for " + name);
                }

                dependency.instance = this.resolveConstructor(dependency.resolver);
            }
        } finally {
            popIndent();
            delete this.resolutionProgress[name];
        }

        return dependency.instance;
    };

    Yok.prototype.resolveDependency = function (name) {
        var module = this.modules[name];
        if (!module) {
            throw new Error("unable to resolve " + name);
        }

        if (module.require) {
            require(module.require);
        }
        return module;
    };

    Yok.prototype.getRegisteredCommandsNames = function (includeDev) {
        var _this = this;
        var modulesNames = _.keys(this.modules);
        var commandsNames = _.filter(modulesNames, function (moduleName) {
            return moduleName.startsWith(util.format("%s.", _this.COMMANDS_NAMESPACE));
        });
        var commands = _.map(commandsNames, function (commandName) {
            return commandName.substr(_this.COMMANDS_NAMESPACE.length + 1);
        });
        if (!includeDev) {
            commands = _.reject(commands, function (command) {
                return command.startsWith("dev-");
            });
        }
        return commands;
    };

    Yok.prototype.getChildrenCommandsNames = function (commandName) {
        return this.hierarchicalCommands[commandName];
    };

    Yok.prototype.createCommandName = function (name) {
        return util.format("%s.%s", this.COMMANDS_NAMESPACE, name);
    };

    Yok.prototype.dispose = function () {
        var _this = this;
        Object.keys(this.modules).forEach(function (moduleName) {
            var instance = _this.modules[moduleName].instance;
            if (instance && instance.dispose && instance !== _this) {
                instance.dispose();
            }
        });
    };
    return Yok;
})();
exports.Yok = Yok;

exports.injector = new Yok();
exports.injector.register("injector", exports.injector);
