var Future = require("fibers/future");
var npm = require("npm");
var path = require("path");
var semver = require("semver");

var helpers = require("./common/helpers");
var constants = require("./constants");
var options = require("./options");

var NodePackageManager = (function () {
    function NodePackageManager($logger, $errors, $httpClient, $staticConfig, $fs) {
        this.$logger = $logger;
        this.$errors = $errors;
        this.$httpClient = $httpClient;
        this.$staticConfig = $staticConfig;
        this.$fs = $fs;
        this.versionsCache = {};
    }
    NodePackageManager.prototype.getCacheRootPath = function () {
        var _this = this;
        return (function () {
            _this.load().wait();
            return npm.cache;
        }).future()();
    };

    NodePackageManager.prototype.addToCache = function (packageName, version) {
        var _this = this;
        return (function () {
            _this.load().wait();
            _this.addToCacheCore(packageName, version).wait();
        }).future()();
    };

    NodePackageManager.prototype.load = function (config) {
        var future = new Future();
        npm.load(config, function (err) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });
        return future;
    };

    NodePackageManager.prototype.install = function (packageName, opts) {
        var _this = this;
        return (function () {
            try  {
                _this.load().wait();

                var packageToInstall = packageName;
                var pathToSave = (opts && opts.pathToSave) || npm.cache;
                var version = (opts && opts.version) || null;
                var isSemanticVersioningDisabled = options.frameworkPath ? true : false;

                if (version) {
                    _this.validateVersion(packageName, version).wait();
                    packageToInstall = packageName + "@" + version;
                }

                _this.installCore(packageToInstall, pathToSave, isSemanticVersioningDisabled).wait();
            } catch (error) {
                _this.$logger.debug(error);
                _this.$errors.fail(NodePackageManager.NPM_LOAD_FAILED);
            }

            var pathToNodeModules = path.join(pathToSave, "node_modules");
            var folders = _this.$fs.readDirectory(pathToNodeModules).wait();
            return path.join(pathToNodeModules, folders[0]);
        }).future()();
    };

    NodePackageManager.prototype.getLatestVersion = function (packageName) {
        var _this = this;
        return (function () {
            var versions = _this.getAvailableVersions(packageName).wait();
            versions = _.sortBy(versions, function (ver) {
                return ver;
            });
            return versions.reverse()[0];
        }).future()();
    };

    NodePackageManager.prototype.installCore = function (packageName, pathToSave, isSemanticVersioningDisabled) {
        var _this = this;
        var currentVersion = this.$staticConfig.version;
        if (!semver.valid(currentVersion)) {
            this.$errors.fail("Invalid version.");
        }

        if (!isSemanticVersioningDisabled) {
            var incrementedVersion = semver.inc(currentVersion, constants.ReleaseType.MINOR);
            if (packageName.indexOf("@") < 0) {
                packageName = packageName + "@<" + incrementedVersion;
            }
        }

        this.$logger.out("Installing ", packageName);

        var future = new Future();
        npm.commands["install"](pathToSave, packageName, function (err, data) {
            if (err) {
                future.throw(err);
            } else {
                _this.$logger.out("Installed ", packageName);
                future.return(data);
            }
        });
        return future;
    };

    NodePackageManager.prototype.addToCacheCore = function (packageName, version) {
        var future = new Future();
        npm.commands["cache"].add(packageName, version, undefined, function (err, data) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });
        return future;
    };

    NodePackageManager.prototype.cacheUnpack = function (packageName, version, unpackTarget) {
        var future = new Future();
        unpackTarget = unpackTarget || path.join(npm.cache, packageName, version, "package");
        npm.commands["cache"].unpack(packageName, version, unpackTarget, function (err, data) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });
        return future;
    };

    NodePackageManager.prototype.getAvailableVersions = function (packageName) {
        var _this = this;
        return (function () {
            if (!_this.versionsCache[packageName]) {
                var url = NodePackageManager.NPM_REGISTRY_URL + packageName;
                var response = _this.$httpClient.httpRequest(url).wait().body;
                var json = JSON.parse(response);
                _this.versionsCache[packageName] = _.keys(json.versions);
            }

            return _this.versionsCache[packageName];
        }).future()();
    };

    NodePackageManager.prototype.validateVersion = function (packageName, version) {
        var _this = this;
        return (function () {
            var versions = _this.getAvailableVersions(packageName).wait();
            if (!_.contains(versions, version)) {
                _this.$errors.fail("Invalid version. Valid versions are: %s", helpers.formatListOfNames(versions, "and"));
            }
        }).future()();
    };
    NodePackageManager.NPM_LOAD_FAILED = "Failed to retrieve data from npm. Please try again a little bit later.";
    NodePackageManager.NPM_REGISTRY_URL = "http://registry.npmjs.org/";
    return NodePackageManager;
})();
exports.NodePackageManager = NodePackageManager;
$injector.register("npm", NodePackageManager);
