var path = require("path");
var shell = require("shelljs");
var util = require("util");
var options = require("./../options");
var constants = require("./../constants");
var hostInfo = require("../common/host-info");

var AndroidProjectService = (function () {
    function AndroidProjectService($androidEmulatorServices, $childProcess, $errors, $fs, $logger, $projectData, $propertiesParser) {
        this.$androidEmulatorServices = $androidEmulatorServices;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$projectData = $projectData;
        this.$propertiesParser = $propertiesParser;
    }
    Object.defineProperty(AndroidProjectService.prototype, "platformData", {
        get: function () {
            return {
                frameworkPackageName: "tns-android",
                normalizedPlatformName: "Android",
                platformProjectService: this,
                emulatorServices: this.$androidEmulatorServices,
                projectRoot: path.join(this.$projectData.platformsDir, "android"),
                deviceBuildOutputPath: path.join(this.$projectData.platformsDir, "android", "bin"),
                validPackageNamesForDevice: [
                    util.format("%s-%s.%s", this.$projectData.projectName, "debug", "apk"),
                    util.format("%s-%s.%s", this.$projectData.projectName, "release", "apk")
                ],
                frameworkFilesExtensions: [".jar", ".dat"]
            };
        },
        enumerable: true,
        configurable: true
    });

    AndroidProjectService.prototype.validate = function () {
        var _this = this;
        return (function () {
            _this.validatePackageName(_this.$projectData.projectId);
            _this.validateProjectName(_this.$projectData.projectName);

            _this.checkAnt().wait() && _this.checkAndroid().wait() && _this.checkJava().wait();
        }).future()();
    };

    AndroidProjectService.prototype.createProject = function (projectRoot, frameworkDir) {
        var _this = this;
        return (function () {
            _this.validateAndroidTarget(frameworkDir);

            var paths = "assets libs res".split(' ').map(function (p) {
                return path.join(frameworkDir, p);
            });
            shell.cp("-R", paths, projectRoot);

            paths = ".project AndroidManifest.xml project.properties".split(' ').map(function (p) {
                return path.join(frameworkDir, p);
            });
            shell.cp("-f", paths, projectRoot);

            var packageName = _this.$projectData.projectId;
            var packageAsPath = packageName.replace(/\./g, path.sep);
            var activityDir = path.join(projectRoot, 'src', packageAsPath);
            _this.$fs.createDirectory(activityDir).wait();
        }).future()();
    };

    AndroidProjectService.prototype.interpolateData = function (projectRoot) {
        var _this = this;
        return (function () {
            var manifestPath = path.join(projectRoot, "AndroidManifest.xml");
            var safeActivityName = _this.$projectData.projectName.replace(/\W/g, '');
            shell.sed('-i', /__PACKAGE__/, _this.$projectData.projectId, manifestPath);
            shell.sed('-i', /__ACTIVITY__/, safeActivityName, manifestPath);
            shell.sed('-i', /__APILEVEL__/, _this.getTarget(projectRoot).wait().split('-')[1], manifestPath);

            var stringsFilePath = path.join(projectRoot, 'res', 'values', 'strings.xml');
            shell.sed('-i', /__NAME__/, _this.$projectData.projectName, stringsFilePath);
            shell.sed('-i', /__TITLE_ACTIVITY__/, _this.$projectData.projectName, stringsFilePath);
            shell.sed('-i', /__NAME__/, _this.$projectData.projectName, path.join(projectRoot, '.project'));
        }).future()();
    };

    AndroidProjectService.prototype.afterCreateProject = function (projectRoot) {
        var _this = this;
        return (function () {
            var targetApi = _this.getTarget(projectRoot).wait();
            _this.$logger.trace("Android target: %s", targetApi);
            _this.runAndroidUpdate(projectRoot, targetApi).wait();
        }).future()();
    };

    AndroidProjectService.prototype.prepareProject = function (platformData) {
        var _this = this;
        return (function () {
            var appSourceDirectory = path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME);
            var assetsDirectory = path.join(platformData.projectRoot, "assets");
            var resDirectory = path.join(platformData.projectRoot, "res");

            shell.cp("-Rf", path.join(appSourceDirectory, "*"), assetsDirectory);

            var appResourcesDirectoryPath = path.join(assetsDirectory, constants.APP_RESOURCES_FOLDER_NAME);
            if (_this.$fs.exists(appResourcesDirectoryPath).wait()) {
                shell.cp("-Rf", path.join(appResourcesDirectoryPath, platformData.normalizedPlatformName, "*"), resDirectory);
                _this.$fs.deleteDirectory(appResourcesDirectoryPath).wait();
            }

            return assetsDirectory;
        }).future()();
    };

    AndroidProjectService.prototype.buildProject = function (projectRoot) {
        var _this = this;
        return (function () {
            var buildConfiguration = options.release ? "release" : "debug";
            var args = _this.getAntArgs(buildConfiguration, projectRoot);
            _this.spawn('ant', args).wait();
        }).future()();
    };

    AndroidProjectService.prototype.isPlatformPrepared = function (projectRoot) {
        return this.$fs.exists(path.join(projectRoot, "assets", constants.APP_FOLDER_NAME));
    };

    AndroidProjectService.prototype.getFrameworkFilesExtensions = function () {
        return [".jar", ".dat"];
    };

    AndroidProjectService.prototype.spawn = function (command, args) {
        if (hostInfo.isWindows()) {
            args.unshift('/s', '/c', command);
            command = 'cmd';
        }

        return this.$childProcess.spawnFromEvent(command, args, "close", { stdio: "inherit" });
    };

    AndroidProjectService.prototype.getAntArgs = function (configuration, projectRoot) {
        var args = [configuration, "-f", path.join(projectRoot, "build.xml")];
        if (configuration === "release") {
            args = args.concat([
                "-Dkey.store", options.keyStorePath,
                "-Dkey.store.password", options.keyStorePassword,
                "-Dkey.alias", options.keyStoreAlias,
                "-Dkey.alias.password", options.keyStoreAliasPassword]);
        }

        return args;
    };

    AndroidProjectService.prototype.runAndroidUpdate = function (projectPath, targetApi) {
        var _this = this;
        return (function () {
            var args = [
                "--path", projectPath,
                "--target", targetApi,
                "--name", _this.$projectData.projectName
            ];

            _this.spawn("android", ['update', 'project'].concat(args)).wait();
        }).future()();
    };

    AndroidProjectService.prototype.validatePackageName = function (packageName) {
        if (!/^[a-zA-Z]+(\.[a-zA-Z0-9][a-zA-Z0-9_]*)+$/.test(packageName)) {
            this.$errors.fail("Package name must look like: com.company.Name");
        }

        if (/\b[Cc]lass\b/.test(packageName)) {
            this.$errors.fail("class is a reserved word");
        }
    };

    AndroidProjectService.prototype.validateProjectName = function (projectName) {
        if (projectName === '') {
            this.$errors.fail("Project name cannot be empty");
        }

        if (/^[0-9]/.test(projectName)) {
            this.$errors.fail("Project name must not begin with a number");
        }
    };

    AndroidProjectService.prototype.validateAndroidTarget = function (frameworkDir) {
        var validTarget = this.getTarget(frameworkDir).wait();
        var output = this.$childProcess.exec('android list targets').wait();
        if (!output.match(validTarget)) {
            this.$errors.fail("Please install Android target %s the Android newest SDK). Make sure you have the latest Android tools installed as well. Run \"android\" from your command-line to install/update any missing SDKs or tools.", validTarget.split('-')[1]);
        }
    };

    AndroidProjectService.prototype.getTarget = function (projectRoot) {
        var _this = this;
        return (function () {
            if (!_this.targetApi) {
                var projectPropertiesFilePath = path.join(projectRoot, "project.properties");

                if (_this.$fs.exists(projectPropertiesFilePath).wait()) {
                    var properties = _this.$propertiesParser.createEditor(projectPropertiesFilePath).wait();
                    _this.targetApi = properties.get("target");
                }
            }

            return _this.targetApi;
        }).future()();
    };

    AndroidProjectService.prototype.checkAnt = function () {
        var _this = this;
        return (function () {
            try  {
                _this.$childProcess.exec("ant -version").wait();
            } catch (error) {
                _this.$errors.fail("Error executing commands 'ant', make sure you have ant installed and added to your PATH.");
            }
        }).future()();
    };

    AndroidProjectService.prototype.checkJava = function () {
        var _this = this;
        return (function () {
            try  {
                _this.$childProcess.exec("java -version").wait();
            } catch (error) {
                _this.$errors.fail("%s\n Failed to run 'java', make sure your java environment is set up.\n Including JDK and JRE.\n Your JAVA_HOME variable is %s", error, process.env.JAVA_HOME);
            }
        }).future()();
    };

    AndroidProjectService.prototype.checkAndroid = function () {
        var _this = this;
        return (function () {
            try  {
                _this.$childProcess.exec('android list targets').wait();
            } catch (error) {
                if (error.match(/command\snot\sfound/)) {
                    _this.$errors.fail("The command \"android\" failed. Make sure you have the latest Android SDK installed, and the \"android\" command (inside the tools/ folder) is added to your path.");
                } else {
                    _this.$errors.fail("An error occurred while listing Android targets");
                }
            }
        }).future()();
    };
    return AndroidProjectService;
})();
$injector.register("androidProjectService", AndroidProjectService);
