var path = require("path");
var shell = require("shelljs");
var util = require("util");
var constants = require("./../constants");
var helpers = require("./../common/helpers");
var options = require("./../options");

var IOSProjectService = (function () {
    function IOSProjectService($projectData, $fs, $childProcess, $errors, $iOSEmulatorServices) {
        this.$projectData = $projectData;
        this.$fs = $fs;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
    }
    Object.defineProperty(IOSProjectService.prototype, "platformData", {
        get: function () {
            return {
                frameworkPackageName: "tns-ios",
                normalizedPlatformName: "iOS",
                platformProjectService: this,
                emulatorServices: this.$iOSEmulatorServices,
                projectRoot: path.join(this.$projectData.platformsDir, "ios"),
                deviceBuildOutputPath: path.join(this.$projectData.platformsDir, "ios", "build", "device"),
                emulatorBuildOutputPath: path.join(this.$projectData.platformsDir, "ios", "build", "emulator"),
                validPackageNamesForDevice: [
                    this.$projectData.projectName + ".ipa"
                ],
                validPackageNamesForEmulator: [
                    this.$projectData.projectName + ".app"
                ],
                frameworkFilesExtensions: [".a", ".h", ".bin"],
                targetedOS: ['darwin']
            };
        },
        enumerable: true,
        configurable: true
    });

    IOSProjectService.prototype.validate = function () {
        var _this = this;
        return (function () {
            try  {
                _this.$childProcess.exec("which xcodebuild").wait();
            } catch (error) {
                _this.$errors.fail("Xcode is not installed. Make sure you have Xcode installed and added to your PATH");
            }

            var xcodeBuildVersion = _this.$childProcess.exec("xcodebuild -version | head -n 1 | sed -e 's/Xcode //'").wait();
            var splitedXcodeBuildVersion = xcodeBuildVersion.split(".");
            if (splitedXcodeBuildVersion.length === 3) {
                xcodeBuildVersion = util.format("%s.%s", splitedXcodeBuildVersion[0], splitedXcodeBuildVersion[1]);
            }

            if (helpers.versionCompare(xcodeBuildVersion, IOSProjectService.XCODEBUILD_MIN_VERSION) < 0) {
                _this.$errors.fail("NativeScript can only run in Xcode version %s or greater", IOSProjectService.XCODEBUILD_MIN_VERSION);
            }
        }).future()();
    };

    IOSProjectService.prototype.createProject = function (projectRoot, frameworkDir) {
        return (function () {
            shell.cp("-R", path.join(frameworkDir, "*"), projectRoot);
        }).future()();
    };

    IOSProjectService.prototype.interpolateData = function (projectRoot) {
        var _this = this;
        return (function () {
            _this.replaceFileName("-Info.plist", path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
            _this.replaceFileName("-Prefix.pch", path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
            _this.replaceFileName(IOSProjectService.XCODE_PROJECT_EXT_NAME, projectRoot).wait();

            var pbxprojFilePath = path.join(projectRoot, _this.$projectData.projectName + IOSProjectService.XCODE_PROJECT_EXT_NAME, "project.pbxproj");
            _this.replaceFileContent(pbxprojFilePath).wait();
        }).future()();
    };

    IOSProjectService.prototype.afterCreateProject = function (projectRoot) {
        var _this = this;
        return (function () {
            _this.$fs.rename(path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER), path.join(projectRoot, _this.$projectData.projectName)).wait();
        }).future()();
    };

    IOSProjectService.prototype.prepareProject = function (platformData) {
        var _this = this;
        return (function () {
            var appSourceDirectory = path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME);
            var appDestinationDirectory = path.join(platformData.projectRoot, _this.$projectData.projectName);
            var resDirectory = path.join(platformData.projectRoot, _this.$projectData.projectName, "Resources", "icons");
            _this.$fs.ensureDirectoryExists(resDirectory).wait();

            shell.cp("-Rf", path.join(appSourceDirectory, "*"), appDestinationDirectory);

            var appResourcesDirectoryPath = path.join(appDestinationDirectory, constants.APP_RESOURCES_FOLDER_NAME);
            if (_this.$fs.exists(appResourcesDirectoryPath).wait()) {
                shell.cp("-Rf", path.join(appResourcesDirectoryPath, platformData.normalizedPlatformName, "*"), resDirectory);
                _this.$fs.deleteDirectory(appResourcesDirectoryPath).wait();
            }

            return appDestinationDirectory;
        }).future()();
    };

    IOSProjectService.prototype.buildProject = function (projectRoot) {
        var _this = this;
        return (function () {
            var basicArgs = [
                "-project", path.join(projectRoot, _this.$projectData.projectName + ".xcodeproj"),
                "-target", _this.$projectData.projectName,
                "-configuration", options.release ? "Release" : "Debug",
                "build"
            ];
            var args = [];

            if (options.device) {
                args = basicArgs.concat([
                    "-xcconfig", path.join(projectRoot, "build.xcconfig"),
                    "-sdk", "iphoneos",
                    "ARCHS=\"armv7\"",
                    "VALID_ARCHS=\"armv7\"",
                    "CONFIGURATION_BUILD_DIR=" + path.join(projectRoot, "build", "device")
                ]);
            } else {
                args = basicArgs.concat([
                    "-sdk", "iphonesimulator",
                    "-arch", "i386",
                    "VALID_ARCHS=\"i386\"",
                    "CONFIGURATION_BUILD_DIR=" + path.join(projectRoot, "build", "emulator")
                ]);
            }

            _this.$childProcess.spawnFromEvent("xcodebuild", args, "exit", { cwd: options, stdio: 'inherit' }).wait();

            if (options.device) {
                var buildOutputPath = path.join(projectRoot, "build", options.device ? "device" : "emulator");

                var xcrunArgs = [
                    "-sdk", "iphoneos",
                    "PackageApplication",
                    "-v", path.join(buildOutputPath, _this.$projectData.projectName + ".app"),
                    "-o", path.join(buildOutputPath, _this.$projectData.projectName + ".ipa")
                ];

                _this.$childProcess.spawnFromEvent("xcrun", xcrunArgs, "exit", { cwd: options, stdio: 'inherit' }).wait();
            }
        }).future()();
    };

    IOSProjectService.prototype.isPlatformPrepared = function (projectRoot) {
        return this.$fs.exists(path.join(projectRoot, this.$projectData.projectName, constants.APP_FOLDER_NAME));
    };

    IOSProjectService.prototype.replaceFileContent = function (file) {
        var _this = this;
        return (function () {
            var fileContent = _this.$fs.readText(file).wait();
            var replacedContent = helpers.stringReplaceAll(fileContent, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, _this.$projectData.projectName);
            _this.$fs.writeFile(file, replacedContent).wait();
        }).future()();
    };

    IOSProjectService.prototype.replaceFileName = function (fileNamePart, fileRootLocation) {
        var _this = this;
        return (function () {
            var oldFileName = IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER + fileNamePart;
            var newFileName = _this.$projectData.projectName + fileNamePart;

            _this.$fs.rename(path.join(fileRootLocation, oldFileName), path.join(fileRootLocation, newFileName)).wait();
        }).future()();
    };
    IOSProjectService.XCODE_PROJECT_EXT_NAME = ".xcodeproj";
    IOSProjectService.XCODEBUILD_MIN_VERSION = "5.0";
    IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER = "__PROJECT_NAME__";
    return IOSProjectService;
})();
$injector.register("iOSProjectService", IOSProjectService);
