var path = require("path");
var shell = require("shelljs");
var util = require("util");
var constants = require("./../constants");
var helpers = require("./../common/helpers");
var options = require("./../options");
var semver = require("semver");

var PlatformService = (function () {
    function PlatformService($devicesServices, $errors, $fs, $logger, $npm, $platformsData, $projectData, $projectDataService, $prompter) {
        this.$devicesServices = $devicesServices;
        this.$errors = $errors;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$npm = $npm;
        this.$platformsData = $platformsData;
        this.$projectData = $projectData;
        this.$projectDataService = $projectDataService;
        this.$prompter = $prompter;
    }
    PlatformService.prototype.addPlatforms = function (platforms) {
        var _this = this;
        return (function () {
            if (!platforms || platforms.length === 0) {
                _this.$errors.fail("No platform specified. Please specify a platform to add");
            }

            var platformsDir = _this.$projectData.platformsDir;
            _this.$fs.ensureDirectoryExists(platformsDir).wait();

            _.each(platforms, function (platform) {
                _this.addPlatform(platform.toLowerCase()).wait();
            });
        }).future()();
    };

    PlatformService.prototype.addPlatform = function (platform) {
        var _this = this;
        return (function () {
            var parts = platform.split("@");
            platform = parts[0];
            var version = parts[1];

            _this.validatePlatform(platform);

            var platformPath = path.join(_this.$projectData.platformsDir, platform);

            if (_this.$fs.exists(platformPath).wait()) {
                _this.$errors.fail("Platform %s already added", platform);
            }

            var platformData = _this.$platformsData.getPlatformData(platform);

            var platformProjectService = platformData.platformProjectService;
            platformProjectService.validate().wait();

            _this.$logger.trace("Creating NativeScript project for the %s platform", platform);
            _this.$logger.trace("Path: %s", platformData.projectRoot);
            _this.$logger.trace("Package: %s", _this.$projectData.projectId);
            _this.$logger.trace("Name: %s", _this.$projectData.projectName);

            _this.$logger.out("Copying template files...");

            var packageToInstall = "";
            var npmOptions = {
                pathToSave: path.join(_this.$projectData.platformsDir, platform)
            };

            if (options.frameworkPath) {
                packageToInstall = options.frameworkPath;
            } else {
                packageToInstall = platformData.frameworkPackageName;
                npmOptions["version"] = version;
            }

            var downloadedPackagePath = _this.$npm.install(packageToInstall, npmOptions).wait();
            var frameworkDir = path.join(downloadedPackagePath, constants.PROJECT_FRAMEWORK_FOLDER_NAME);

            try  {
                _this.addPlatformCore(platformData, frameworkDir).wait();
            } catch (err) {
                _this.$fs.deleteDirectory(platformPath).wait();
                throw err;
            }

            _this.$logger.out("Project successfully created.");
        }).future()();
    };

    PlatformService.prototype.addPlatformCore = function (platformData, frameworkDir) {
        var _this = this;
        return (function () {
            platformData.platformProjectService.createProject(platformData.projectRoot, frameworkDir).wait();
            var installedVersion = _this.$fs.readJson(path.join(frameworkDir, "../", "package.json")).wait().version;

            _this.$fs.deleteDirectory(path.join(frameworkDir, "../../")).wait();

            platformData.platformProjectService.interpolateData(platformData.projectRoot).wait();
            platformData.platformProjectService.afterCreateProject(platformData.projectRoot).wait();

            _this.$projectDataService.initialize(_this.$projectData.projectDir);
            _this.$projectDataService.setValue(platformData.frameworkPackageName, { version: installedVersion }).wait();
        }).future()();
    };

    PlatformService.prototype.getInstalledPlatforms = function () {
        var _this = this;
        return (function () {
            if (!_this.$fs.exists(_this.$projectData.platformsDir).wait()) {
                return [];
            }

            var subDirs = _this.$fs.readDirectory(_this.$projectData.platformsDir).wait();
            return _.filter(subDirs, function (p) {
                return _this.$platformsData.platformsNames.indexOf(p) > -1;
            });
        }).future()();
    };

    PlatformService.prototype.getAvailablePlatforms = function () {
        var _this = this;
        return (function () {
            var installedPlatforms = _this.getInstalledPlatforms().wait();
            return _.filter(_this.$platformsData.platformsNames, function (p) {
                return installedPlatforms.indexOf(p) < 0 && _this.isPlatformSupportedForOS(p);
            });
        }).future()();
    };

    PlatformService.prototype.getPreparedPlatforms = function () {
        var _this = this;
        return (function () {
            return _.filter(_this.$platformsData.platformsNames, function (p) {
                return _this.isPlatformPrepared(p).wait();
            });
        }).future()();
    };

    PlatformService.prototype.preparePlatform = function (platform) {
        var _this = this;
        return (function () {
            _this.validatePlatformInstalled(platform);
            platform = platform.toLowerCase();

            var platformData = _this.$platformsData.getPlatformData(platform);
            var platformProjectService = platformData.platformProjectService;

            var appFilesLocation = platformProjectService.prepareProject(platformData).wait();

            var appDirectoryPath = path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME);
            var contents = _this.$fs.readDirectory(appDirectoryPath).wait();
            var files = [];

            _.each(contents, function (d) {
                var fsStat = _this.$fs.getFsStats(path.join(appDirectoryPath, d)).wait();
                if (fsStat.isDirectory() && d !== constants.APP_RESOURCES_FOLDER_NAME) {
                    _this.processPlatformSpecificFiles(platform, helpers.enumerateFilesInDirectorySync(path.join(appFilesLocation, d))).wait();
                } else if (fsStat.isFile()) {
                    files.push(path.join(appFilesLocation, d));
                }
            });

            _this.processPlatformSpecificFiles(platform, files).wait();

            _this.$logger.out("Project successfully prepared");
        }).future()();
    };

    PlatformService.prototype.buildPlatform = function (platform) {
        var _this = this;
        return (function () {
            _this.validatePlatformInstalled(platform);
            platform = platform.toLowerCase();

            var platformData = _this.$platformsData.getPlatformData(platform);
            platformData.platformProjectService.buildProject(platformData.projectRoot).wait();
            _this.$logger.out("Project successfully built");
        }).future()();
    };

    PlatformService.prototype.runPlatform = function (platform) {
        var _this = this;
        return (function () {
            _this.validatePlatformInstalled(platform);
            platform = platform.toLowerCase();

            _this.preparePlatform(platform).wait();
            if (options.emulator) {
                _this.deployOnEmulator(platform).wait();
            } else {
                _this.deployOnDevice(platform).wait();
            }
        }).future()();
    };

    PlatformService.prototype.removePlatforms = function (platforms) {
        var _this = this;
        return (function () {
            if (!platforms || platforms.length === 0) {
                _this.$errors.fail("No platform specified. Please specify a platform to remove");
            }

            _.each(platforms, function (platform) {
                _this.validatePlatformInstalled(platform);

                var platformDir = path.join(_this.$projectData.platformsDir, platform);
                _this.$fs.deleteDirectory(platformDir).wait();
            });
        }).future()();
    };

    PlatformService.prototype.updatePlatforms = function (platforms) {
        var _this = this;
        return (function () {
            if (!platforms || platforms.length === 0) {
                _this.$errors.fail("No platform specified. Please specify a platform to remove");
            }

            _.each(platforms, function (platform) {
                var parts = platform.split("@");
                platform = parts[0].toLowerCase();
                var version = parts[1];

                _this.validatePlatformInstalled(platform);
                _this.updatePlatform(platform, version).wait();
            });
        }).future()();
    };

    PlatformService.prototype.deployOnDevice = function (platform) {
        var _this = this;
        return (function () {
            _this.validatePlatformInstalled(platform);
            platform = platform.toLowerCase();

            var platformData = _this.$platformsData.getPlatformData(platform);

            var cachedDeviceOption = options.device;
            options.device = true;
            _this.buildPlatform(platform).wait();
            options.device = cachedDeviceOption;

            var packageFile = _this.getLatestApplicationPackageForDevice(platformData).wait().packageName;
            _this.$logger.out("Using ", packageFile);

            _this.$devicesServices.initialize(platform, options.device).wait();
            var action = function (device) {
                return device.deploy(packageFile, _this.$projectData.projectId);
            };
            _this.$devicesServices.execute(action).wait();
        }).future()();
    };

    PlatformService.prototype.deployOnEmulator = function (platform) {
        var _this = this;
        return (function () {
            _this.validatePlatformInstalled(platform);
            platform = platform.toLowerCase();

            var platformData = _this.$platformsData.getPlatformData(platform);
            var emulatorServices = platformData.emulatorServices;

            emulatorServices.checkAvailability().wait();

            _this.buildPlatform(platform).wait();

            var packageFile = _this.getLatestApplicationPackageForEmulator(platformData).wait().packageName;
            _this.$logger.out("Using ", packageFile);

            var logFilePath = path.join(platformData.projectRoot, _this.$projectData.projectName, "emulator.log");

            emulatorServices.startEmulator(packageFile, { image: options.emulator, stderrFilePath: logFilePath, stdoutFilePath: logFilePath }).wait();
        }).future()();
    };

    PlatformService.prototype.validatePlatform = function (platform) {
        if (!platform) {
            this.$errors.fail("No platform specified.");
        }

        platform = platform.toLowerCase();

        if (!this.isValidPlatform(platform)) {
            this.$errors.fail("Invalid platform %s. Valid platforms are %s.", platform, helpers.formatListOfNames(this.$platformsData.platformsNames));
        }

        if (!this.isPlatformSupportedForOS(platform)) {
            this.$errors.fail("Applications for platform %s can not be built on this OS - %s", platform, process.platform);
        }
    };

    PlatformService.prototype.validatePlatformInstalled = function (platform) {
        this.validatePlatform(platform);

        if (!this.isPlatformInstalled(platform).wait()) {
            this.$errors.fail("The platform %s is not added to this project. Please use 'tns platform add <platform>'", platform);
        }
    };

    PlatformService.prototype.isValidPlatform = function (platform) {
        return this.$platformsData.getPlatformData(platform);
    };

    PlatformService.prototype.isPlatformSupportedForOS = function (platform) {
        var targetedOS = this.$platformsData.getPlatformData(platform).targetedOS;

        if (!targetedOS || targetedOS.indexOf("*") >= 0 || targetedOS.indexOf(process.platform) >= 0) {
            return true;
        }

        return false;
    };

    PlatformService.prototype.isPlatformInstalled = function (platform) {
        var _this = this;
        return (function () {
            return _this.$fs.exists(path.join(_this.$projectData.platformsDir, platform)).wait();
        }).future()();
    };

    PlatformService.prototype.isPlatformPrepared = function (platform) {
        var platformData = this.$platformsData.getPlatformData(platform);
        return platformData.platformProjectService.isPlatformPrepared(platformData.projectRoot);
    };

    PlatformService.parsePlatformSpecificFileName = function (fileName, platforms) {
        var regex = util.format("^(.+?)\.(%s)(\..+?)$", platforms.join("|"));
        var parsed = fileName.toLowerCase().match(new RegExp(regex, "i"));
        if (parsed) {
            return {
                platform: parsed[2],
                onDeviceName: parsed[1] + parsed[3]
            };
        }
        return undefined;
    };

    PlatformService.prototype.processPlatformSpecificFiles = function (platform, files) {
        var _this = this;
        return (function () {
            _.each(files, function (fileName) {
                var platformInfo = PlatformService.parsePlatformSpecificFileName(path.basename(fileName), _this.$platformsData.platformsNames);
                var shouldExcludeFile = platformInfo && platformInfo.platform !== platform;
                if (shouldExcludeFile) {
                    _this.$fs.deleteFile(fileName).wait();
                } else if (platformInfo && platformInfo.onDeviceName) {
                    _this.$fs.rename(fileName, path.join(path.dirname(fileName), platformInfo.onDeviceName)).wait();
                }
            });
        }).future()();
    };

    PlatformService.prototype.getApplicationPackages = function (buildOutputPath, validPackageNames) {
        var _this = this;
        return (function () {
            var candidates = _this.$fs.readDirectory(buildOutputPath).wait();
            var packages = _.filter(candidates, function (candidate) {
                return _.contains(validPackageNames, candidate);
            }).map(function (currentPackage) {
                currentPackage = path.join(buildOutputPath, currentPackage);

                return {
                    packageName: currentPackage,
                    time: _this.$fs.getFsStats(currentPackage).wait().mtime
                };
            });

            return packages;
        }).future()();
    };

    PlatformService.prototype.getLatestApplicationPackage = function (buildOutputPath, validPackageNames) {
        var _this = this;
        return (function () {
            var packages = _this.getApplicationPackages(buildOutputPath, validPackageNames).wait();
            if (packages.length === 0) {
                var packageExtName = path.extname(validPackageNames[0]);
                _this.$errors.fail("No %s found in %s directory", packageExtName, buildOutputPath);
            }

            packages = _.sortBy(packages, function (pkg) {
                return pkg.time;
            }).reverse();

            return packages[0];
        }).future()();
    };

    PlatformService.prototype.getLatestApplicationPackageForDevice = function (platformData) {
        return this.getLatestApplicationPackage(platformData.deviceBuildOutputPath, platformData.validPackageNamesForDevice);
    };

    PlatformService.prototype.getLatestApplicationPackageForEmulator = function (platformData) {
        return this.getLatestApplicationPackage(platformData.emulatorBuildOutputPath || platformData.deviceBuildOutputPath, platformData.validPackageNamesForEmulator || platformData.validPackageNamesForDevice);
    };

    PlatformService.prototype.updatePlatform = function (platform, version) {
        var _this = this;
        return (function () {
            var platformData = _this.$platformsData.getPlatformData(platform);

            _this.$projectDataService.initialize(_this.$projectData.projectDir);
            var data = _this.$projectDataService.getValue(platformData.frameworkPackageName).wait();
            var currentVersion = data && data.version ? data.version : "0.2.0";
            var newVersion = version || _this.$npm.getLatestVersion(platformData.frameworkPackageName).wait();

            if (!semver.valid(newVersion)) {
                _this.$errors.fail("The version %s is not valid. The version should consists from 3 parts seperated by dot.", newVersion);
            }

            if (semver.gt(currentVersion, newVersion)) {
                var isUpdateConfirmed = _this.$prompter.confirm("You are going to update to lower version. Are you sure?", function () {
                    return "n";
                }).wait();
                if (isUpdateConfirmed) {
                    _this.updatePlatformCore(platformData, currentVersion, newVersion).wait();
                }
            } else if (semver.eq(currentVersion, newVersion)) {
                _this.$errors.fail("Current and new version are the same.");
            } else {
                _this.updatePlatformCore(platformData, currentVersion, newVersion).wait();
            }
        }).future()();
    };

    PlatformService.prototype.updatePlatformCore = function (platformData, currentVersion, newVersion) {
        var _this = this;
        return (function () {
            var oldFrameworkFiles = _this.getFrameworkFiles(platformData, currentVersion).wait();
            _.each(oldFrameworkFiles, function (file) {
                _this.$fs.deleteFile(path.join(platformData.projectRoot, file)).wait();
            });

            var newFrameworkFiles = _this.getFrameworkFiles(platformData, newVersion).wait();
            var cacheDirectoryPath = _this.getNpmCacheDirectoryCore(platformData.frameworkPackageName, newVersion).wait();
            _.each(newFrameworkFiles, function (file) {
                shell.cp("-f", path.join(cacheDirectoryPath, file), path.join(platformData.projectRoot, file));
            });

            _this.$projectDataService.initialize(_this.$projectData.projectDir);
            _this.$projectDataService.setValue(platformData.frameworkPackageName, { version: newVersion }).wait();

            _this.$logger.out("Successfully updated to version ", newVersion);
        }).future()();
    };

    PlatformService.prototype.getFrameworkFiles = function (platformData, version) {
        var _this = this;
        return (function () {
            var npmCacheDirectoryPath = _this.getNpmCacheDirectory(platformData.frameworkPackageName, version).wait();
            var allFiles = helpers.enumerateFilesInDirectorySync(npmCacheDirectoryPath);
            var filteredFiles = _.filter(allFiles, function (file) {
                return _.contains(platformData.frameworkFilesExtensions, path.extname(file));
            });
            var relativeToCacheFiles = _.map(filteredFiles, function (file) {
                return file.substr(npmCacheDirectoryPath.length);
            });

            return relativeToCacheFiles;
        }).future()();
    };

    PlatformService.prototype.getNpmCacheDirectory = function (packageName, version) {
        var _this = this;
        return (function () {
            var npmCacheDirectoryPath = _this.getNpmCacheDirectoryCore(packageName, version).wait();

            if (!_this.$fs.exists(npmCacheDirectoryPath).wait()) {
                _this.$npm.addToCache(packageName, version).wait();
            }

            return npmCacheDirectoryPath;
        }).future()();
    };

    PlatformService.prototype.getNpmCacheDirectoryCore = function (packageName, version) {
        var _this = this;
        return (function () {
            var npmCacheRoot = _this.$npm.getCacheRootPath().wait();
            return path.join(npmCacheRoot, packageName, version, "package");
        }).future()();
    };
    return PlatformService;
})();
exports.PlatformService = PlatformService;
$injector.register("platformService", PlatformService);
