var constants = require("./../constants");

var options = require("./../options");
var osenv = require("osenv");
var path = require("path");
var shell = require("shelljs");

var ProjectService = (function () {
    function ProjectService($errors, $fs, $logger, $projectDataService, $projectHelper, $projectNameValidator, $projectTemplatesService) {
        this.$errors = $errors;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$projectDataService = $projectDataService;
        this.$projectHelper = $projectHelper;
        this.$projectNameValidator = $projectNameValidator;
        this.$projectTemplatesService = $projectTemplatesService;
    }
    ProjectService.prototype.createProject = function (projectName) {
        var _this = this;
        return (function () {
            if (!projectName) {
                _this.$errors.fail("You must specify <App name> when creating a new project.");
            }
            _this.$projectNameValidator.validate(projectName);

            var projectId = options.appid || _this.$projectHelper.generateDefaultAppId(projectName, "org.nativescript");

            var projectDir = path.join(path.resolve(options.path || "."), projectName);
            _this.$fs.createDirectory(projectDir).wait();

            var customAppPath = _this.getCustomAppPath();
            if (customAppPath) {
                customAppPath = path.resolve(customAppPath);
            }

            if (_this.$fs.exists(projectDir).wait() && !_this.$fs.isEmptyDir(projectDir).wait()) {
                _this.$errors.fail("Path already exists and is not empty %s", projectDir);
            }

            _this.$logger.trace("Creating a new NativeScript project with name %s and id %s at location %s", projectName, projectId, projectDir);

            var appDirectory = path.join(projectDir, constants.APP_FOLDER_NAME);
            var appPath = null;

            if (customAppPath) {
                _this.$logger.trace("Using custom app from %s", customAppPath);

                var relativePathFromSourceToTarget = path.relative(customAppPath, appDirectory);

                if (relativePathFromSourceToTarget !== appDirectory) {
                    var doesRelativePathGoUpAtLeastOneDir = relativePathFromSourceToTarget.split(path.sep)[0] === "..";
                    if (!doesRelativePathGoUpAtLeastOneDir) {
                        _this.$errors.fail("Project dir %s must not be created at/inside the template used to create the project %s.", projectDir, customAppPath);
                    }
                }
                _this.$logger.trace("Copying custom app into %s", appDirectory);
                appPath = customAppPath;
            } else {
                _this.$logger.trace("Using NativeScript hello world application");
                var defaultTemplatePath = _this.$projectTemplatesService.defaultTemplatePath.wait();
                _this.$logger.trace("Copying NativeScript hello world application into %s", appDirectory);
                appPath = defaultTemplatePath;
            }

            try  {
                _this.createProjectCore(projectDir, appPath, projectId, false).wait();
            } catch (err) {
                _this.$fs.deleteDirectory(projectDir).wait();
                throw err;
            }

            _this.$logger.out("Project %s was successfully created", projectName);
        }).future()();
    };

    ProjectService.prototype.createProjectCore = function (projectDir, appPath, projectId, symlink) {
        var _this = this;
        return (function () {
            if (!_this.$fs.exists(projectDir).wait()) {
                _this.$fs.createDirectory(projectDir).wait();
            }
            if (symlink) {
            } else {
                var appDir = path.join(projectDir, constants.APP_FOLDER_NAME);
                _this.$fs.createDirectory(appDir).wait();
                shell.cp('-R', path.join(appPath, "*"), appDir);
            }
            _this.createBasicProjectStructure(projectDir, projectId).wait();
        }).future()();
    };

    ProjectService.prototype.createBasicProjectStructure = function (projectDir, projectId) {
        var _this = this;
        return (function () {
            _this.$fs.createDirectory(path.join(projectDir, "platforms")).wait();

            _this.$projectDataService.initialize(projectDir);
            _this.$projectDataService.setValue("id", projectId).wait();
        }).future()();
    };

    ProjectService.prototype.getCustomAppPath = function () {
        var customAppPath = options["copy-from"] || options["link-to"];
        if (customAppPath) {
            if (customAppPath.indexOf("http://") === 0) {
                this.$errors.fail("Only local paths for custom app are supported.");
            }

            if (customAppPath.substr(0, 1) === '~') {
                customAppPath = path.join(osenv.home(), customAppPath.substr(1));
            }
        }

        return customAppPath;
    };
    return ProjectService;
})();
exports.ProjectService = ProjectService;
$injector.register("projectService", ProjectService);
